/******************************************************************************
 * 
 * Capacitated m-Ring Star
 * 
 * Solución de m-anillos estrella con capacidades
 * 
 ******************************************************************************/

#ifndef __CMRS
#define __CMRS

#include <vector>
#include <string>
#include <set>
#include "CMRSPException.h"

using namespace std;



class CMRS 
{
    public:
        typedef pair<string, string> Arc;
        typedef pair<Arc, double> WeightedArc;
        
        CMRS(int n_rings, int ring_capacity);
        void depot(const string& depot_label) throw(CMRSPException);
        void add_customer(const string& customer_label) throw(CMRSPException);
        void add_steiner(const string& steiner_label) throw(CMRSPException);
        void add_ring_connection(const string& source_label, const string& target_label, double weight = 1.0) throw(CMRSPException);
        void add_star_connection(const string& source_label, const string& target_label, double weight = 1.0) throw(CMRSPException);
        string get_depot() const;
        set<string> get_customers() const;
        set<string> get_steiners() const;
        set<WeightedArc> get_ring_connections() const;
        set<WeightedArc> get_star_connections() const;
        bool validate() const;
        void save(const string& filename) throw(CMRSPException);
        void save_dot(const string& filename) throw(CMRSPException);
        void print();
        
    private:
        int n_rings;
        int ring_capacity;
        string depot_label;
        set<string> customers_labels;
        set<string> steiners_labels;
        set<WeightedArc> ring_connections;
        set<WeightedArc> star_connections;

};

#endif
