/******************************************************************************
 * 
 * Capacitated m-Ring Star Problem
 * 
 * Instancia del problema de m-anillos estrella con capacidades
 * 
 ******************************************************************************/

#ifndef __CMRSP
#define __CMRSP

#include <vector>
#include <string>
#include <ilcplex/cplex.h>
#include "CMRS.h"
#include "CMRSPException.h"
#include "ExecParam.h"

using namespace std;


class CMRSP 
{
    public:
        CMRSP(const ExecParam&) throw(CMRSPException);
        ~CMRSP();
        void optimize() throw(CMRSPException);
        CMRS solution() throw(CMRSPException);
        void save_log() throw(CMRSPException);
    
    private:
        void create_instance(const string& filename) throw(CMRSPException);
        void load_parameters(const string& filename) throw(CMRSPException);
        void build_instance() throw(CMRSPException);
        void initial_heuristic() throw(CMRSPException);
        void initial_heuristic1() throw(CMRSPException);
        void initial_heuristic2() throw(CMRSPException);
        void initial_heuristic3() throw(CMRSPException);
        CPXFILEptr create_logfile(const string& filename) throw(CMRSPException);
        void mip_optimize() throw(CMRSPException);
        void mip_solution() throw(CMRSPException);        
        static int CPXPUBLIC mycutcallback (CPXCENVptr env, void *cbdata, int wherefrom, void *cbhandle, int *useraction_p);
        static bool separate1(double* x, void *cbdata, int wherefrom) throw(CMRSPException); // Componentes
        static bool apply_cuts1(double* x, void *cbdata, int wherefrom) throw(CMRSPException); // Componentes
        static bool apply_cuts4(double* x, void *cbdata, int wherefrom) throw(CMRSPException); // Componentes
        //static bool apply_cuts2(double* x, void *cbdata, int wherefrom) throw(CMRSPException); // Ciclos
        //static bool apply_cuts3(double* x, void *cbdata, int wherefrom) throw(CMRSPException); // Estructura que involucra arcos anillo y arcos estrella
        static CMRS solution(double* x);
        
};

string label_x(int index, int i);
string label_y(int index, int i);
int var_x(int i, int j);
int var_y(int i, int j);
int var_z(int i);
int var_f(int i);

#endif
