#ifndef __EXECMANAGER
#define __EXECMANAGER

#include <iostream>
#include <string>
#include <vector>
#include "ExecParam.h"
#include "CMRSPbranching.h"
#include "CMRS.h"
#include "CMRSRep.h"
#include "HeuristicaInicial.h"
#include "HeuristicaInicialClusterRoute1.h"
#include "Separacion.h"
#include "SeparacionComponentes.h"
#include "SeparacionAnilloOEstrella.h"
#include "SeparacionFlujoMaximo.h"
#include "SeparacionBusquedaLocal.h"
using namespace std;

void parameters_sets(vector<ExecParam>& parameters);

int main(int argc, char** argv)
{
    cout << "Capacitated m-Ring-Star Problem Solver" << endl;

    if (argc < 2)
    {
        cout << "Uso: opt instancia_1.dat [instancia_2.dat | ... | instancia_n.dat]" << endl;
        return -1;
    }
    
    vector<ExecParam> parameters_families;
    parameters_sets(parameters_families);
    
    try
    {
        for (int i = 1; i < argc; i++)
        {
            for (int j = 0; j < parameters_families.size(); j++)
            {
                ExecParam param = parameters_families[j];
                param.filename = string(argv[1]);
                CMRSRep cmrs_rep = CMRSRep(param.filename);
                
                HeuristicaInicial* heuristica_inicial = new HeuristicaInicialClusterRoute1(&cmrs_rep);
                Separacion* metodo_separacion_componentes = new SeparacionComponentes(&cmrs_rep);
                Separacion* metodo_separacion_anillo_o_estrella = new SeparacionAnilloOEstrella(&cmrs_rep);
                //Separacion* metodo_separacion = new SeparacionFlujoMaximo(&cmrs_rep);
                //Separacion* metodo_separacion = new SeparacionBusquedaLocal(&cmrs_rep);
                
                CMRSP cmrsp(&cmrs_rep, param);
                cmrsp.set_heuristica_inicial(heuristica_inicial);
                cmrsp.add_metodo_separacion(metodo_separacion_componentes);
                cmrsp.add_metodo_separacion(metodo_separacion_anillo_o_estrella);
                cmrsp.optimize();
                cmrsp.save_log();
                CMRS cmrs = cmrsp.solution();
                cmrs.save(string(argv[1])+".sol");
                
                delete heuristica_inicial;
                delete metodo_separacion_componentes;
                delete metodo_separacion_anillo_o_estrella;
            }
        }

    }
    catch(CMRSPException& e)
    {        
        cout << "**** ERROR ****" << endl << e.what() << endl;
    }

    return 0;
}



void parameters_sets(vector<ExecParam>& parameters)
{
    ExecParam param;

    // Todo deshabilitado (default cplex)
    param.node_sel = -1;
    param.var_sel = -1;
    param.var_priority = -1;
    param.mip_emphasis = -1;
    param.time_limit = 1800;
    param.logging_interval = 100;
    param.initial_heuristic = false;
    param.clique_cuts = true;
    param.cover_cuts = true;
    param.disjuntive_cuts = true;
    param.flowcover_cuts = true;
    param.flowpath_cuts = true;
    param.fractional_cuts = true;
    param.gubcover_cuts = true;
    param.mir_cuts = true;
    param.branching_1 = false;
    param.cutting_plane_1 = false;
    param.cutting_plane_2 = false;
    param.cutting_plane_3 = false;
    param.cutting_plane_4 = false;
    param.cutting_plane_levels = 10000;
    param.primal_heuristic = false;
    param.continuous_vars = false;
    param.sos = false;
    parameters.push_back(param);

    // Todo deshabilitado salvo Heuristica inicial (prueba heuristica inicial)
    param.node_sel = -1;
    param.var_sel = -1;
    param.var_priority = -1;
    param.mip_emphasis = -1;
    param.time_limit = 1800;
    param.logging_interval = 100;
    param.initial_heuristic = true;
    param.clique_cuts = true;
    param.cover_cuts = true;
    param.disjuntive_cuts = true;
    param.flowcover_cuts = true;
    param.flowpath_cuts = true;
    param.fractional_cuts = true;
    param.gubcover_cuts = true;
    param.mir_cuts = true;
    param.branching_1 = false;
    param.cutting_plane_1 = false;
    param.cutting_plane_2 = false;
    param.cutting_plane_3 = false;
    param.cutting_plane_4 = false;
    param.cutting_plane_levels = 10000;
    param.primal_heuristic = false;
    param.continuous_vars = false;
    param.sos = false;
    parameters.push_back(param);


//     // Mejor combinacion encontrada
//     //for (int node_sel = 0; node_sel <= 3; node_sel++)
//     for (int node_sel = 1; node_sel <= 1; node_sel++)
//     {
//     //for (int mip_emphasis = 0; mip_emphasis <= 4; mip_emphasis++)
//     for (int mip_emphasis = 1; mip_emphasis <= 1; mip_emphasis++)
//     {
//     //for (int var_sel = 2 /*-1*/; var_sel <= 4; var_sel++)
//     for (int var_sel = 4; var_sel <= 4; var_sel++)
//     {
//     for (int var_priority = 3; var_priority <= 3; var_priority++)
//     {
//         param.node_sel = node_sel;
//         param.var_sel = var_sel;
//         param.var_priority = var_priority;
//         param.mip_emphasis = mip_emphasis;
//         param.time_limit = 1800;
//         param.logging_interval = 100;
//         param.initial_heuristic = true;
//         param.clique_cuts = true;
//         param.cover_cuts = true;
//         param.disjuntive_cuts = true;
//         param.flowcover_cuts = true;
//         param.flowpath_cuts = true;
//         param.fractional_cuts = true;
//         param.gubcover_cuts = true;
//         param.mir_cuts = true;
//         param.branching_1 = true;
//         param.cutting_plane_1 = true;
//         param.cutting_plane_2 = false;
//         param.cutting_plane_3 = false;
//         param.cutting_plane_4 = false;
//         param.cutting_plane_levels = 10000;
//         param.primal_heuristic = false;
//         param.continuous_vars = false;
//         param.sos = false;
//         parameters.push_back(param);
//     }
//     }
//     }
//     }

}




#endif
