/******************************************************************************
 * 
 * Capacitated m-Ring Star Problem
 * 
 * Instancia del problema de m-anillos estrella con capacidades
 * 
 ******************************************************************************/

#ifndef __CMRSP
#define __CMRSP

#include <vector>
#include <string>
#include <ilcplex/cplex.h>
#include "CMRS.h"
#include "CMRSPException.h"
#include "ExecParam.h"
#include "CMRSRep.h"
#include "HeuristicaInicial.h"
#include "Separacion.h"

using namespace std;


class CMRSP 
{
    public:
        CMRSP(CMRSRep* cmrs_rep, const ExecParam&) throw(CMRSPException);
        ~CMRSP();
        void set_heuristica_inicial(HeuristicaInicial* heuristica_inicial);
        void add_metodo_separacion(Separacion* metodo_separacion);
        void optimize() throw(CMRSPException);
        CMRS solution() throw(CMRSPException);
        void save_log() throw(CMRSPException);
    
    private:
        void create_instance(const string& filename) throw(CMRSPException);
        void load_parameters(const string& filename) throw(CMRSPException);
        void build_instance() throw(CMRSPException);
        CPXFILEptr create_logfile(const string& filename) throw(CMRSPException);
        void mip_optimize() throw(CMRSPException);
        void mip_solution() throw(CMRSPException);        
        static CMRS solution(double* x);
        static int CPXPUBLIC mycutcallback (CPXCENVptr env, void *cbdata, int wherefrom, void *cbhandle, int *useraction_p);
//         static int CPXPUBLIC usersetbranch (CPXCENVptr env, void *cbdata, int wherefrom, void *cbhandle, int brtype, int sos, int nodes,
//                                             int bdcnt, const double *nodeest, const int *nodebeg,
//                                             const int *indices, const char *lu, const int *bd,
//                                             int *useraction_p);

                
};

#endif
