#include "CMRSRep.h"

#include <vector>
#include <string>
#include <cassert>
#include <fstream>
#include <iostream>
using namespace std;


CMRSRep::CMRSRep(const string& filename)
{
    /***************************************************************************
     * 
     * Lee los parametros de la instancia:
     * cantidad de anillos, capacidad de anillo, matrices de costos de conexión
     * y de costos de ruteo
     * 
     ***************************************************************************/
    int i, j;

    ifstream ifs(filename.c_str());
    assert(ifs);
    
    ifs >> _n_rings;         // cantidad de anillos
    ifs >> _ring_capacity;   // capacidad de cada anillo
    ifs >> _n_customers;     // cantidad de nodos cliente
    ifs >> _n_steiners;      // cantidad de nodos de transicion
    ifs >> _depot0_label;    // nombre del nodo de deposito (salida/inicial)
    ifs >> _depot1_label;    // nombre del nodo de deposito (llegada/final)

    _V = _n_customers + _n_steiners + 2;
    _V1 = _n_customers + _n_steiners;
    _V2 = _V*_V;
    
    _customers_labels = vector<string>(_n_customers);   // nombres de los nodos cliente
    for (i = 0; i < _n_customers; i++)
        ifs >> _customers_labels[i];
    
    _steiners_labels = vector<string>(_n_steiners);    // nombres de los nodos de transicion
    for (i = 0; i < _n_steiners; i++)
        ifs >> _steiners_labels[i];
    
    _connection_cost = vector<vector<double> >(_V);    // matriz de costos de conexion
    for (i = 0; i < _V; i++)
    {
        _connection_cost[i] = vector<double>(_V);
        for (j = 0; j < _V; j++)
        {
            ifs >> _connection_cost[i][j];
        }
    }

    _routing_cost = vector<vector<double> >(_n_customers);        // matriz de costos de ruteo
    for (i = 0; i < _n_customers; i++)
    {
        _routing_cost[i] = vector<double>(_V1);
        for (j = 0; j < _V1; j++)
        {
            ifs >> _routing_cost[i][j];
        }
    }

    ifs.close();    

    _index_x.D0 = 0;
    _index_x.D1 = 1;
    _index_x.CUSTOMER = 2;
    _index_x.STEINER = 2 + _n_customers;
    _index_y.CUSTOMER = 0;
    _index_y.STEINER = _n_customers;
    _index_z.CUSTOMER = 0;
    _index_z.STEINER = _n_customers;
    _index_f.CUSTOMER = 0;
    _index_f.STEINER = _n_customers;
    _index_p.CUSTOMER = 0;
    _index_p.STEINER = _n_customers;
    _index_connection_cost.D0 = 0;
    _index_connection_cost.D1 = 1;
    _index_connection_cost.CUSTOMER = 2;
    _index_connection_cost.STEINER = 2 + _n_customers;
    _index_routing_cost.CUSTOMER = 0;
    _index_routing_cost.STEINER = _n_customers;
    _connection_cost = _connection_cost;
    _routing_cost = _routing_cost;

}


// CMRSRep::CMRSRep(int n_customers, int n_steiners, int n_rings, int ring_capacity, vector<vector<double> > connection_cost, vector<vector<double> > routing_cost)
// {
//     _n_customers = n_customers;
//     _n_steiners = n_steiners;
//     _n_rings = n_rings;
//     _ring_capacity = ring_capacity;
//     _V = n_customers + n_steiners + 2;
//     _V1 = n_customers + n_steiners;
//     _V2 = _V * _V;
//     _index_x.D0 = 0;
//     _index_x.D1 = 1;
//     _index_x.CUSTOMER = 2;
//     _index_x.STEINER = 2 + n_customers;
//     _index_y.CUSTOMER = 0;
//     _index_y.STEINER = n_customers;
//     _index_z.CUSTOMER = 0;
//     _index_z.STEINER = n_customers;
//     _index_f.CUSTOMER = 0;
//     _index_f.STEINER = n_customers;
//     _index_p.CUSTOMER = 0;
//     _index_p.STEINER = n_customers;
//     _index_connection_cost.D0 = 0;
//     _index_connection_cost.D1 = 1;
//     _index_connection_cost.CUSTOMER = 2;
//     _index_connection_cost.STEINER = 2 + n_customers;
//     _index_routing_cost.CUSTOMER = 0;
//     _index_routing_cost.STEINER = n_customers;
//     _connection_cost = connection_cost;
//     _routing_cost = routing_cost;
// }

string CMRSRep::filename()
{
    return _filename;
}

int CMRSRep::n_customers()
{
    return _n_customers;
}

int CMRSRep::n_steiners()
{
    return _n_steiners;
}

int CMRSRep::n_rings()
{
    return _n_rings;
}

int CMRSRep::ring_capacity()
{
    return _ring_capacity;
}

int CMRSRep::V()
{
    return _V;
}

int CMRSRep::V1()
{
    return _V1;
}

int CMRSRep::V2()
{
    return _V2;
}

int CMRSRep::var_x(int i, int j)
{
    return i*_V + j;
}

int CMRSRep::var_y(int i, int j)
{
    return _V2 + i*_V1 + j;
}

int CMRSRep::var_z(int i)
{
    return _V2 + _n_customers*_V1 + i;
}

int CMRSRep::var_f(int i)
{
    return _V2 + _n_customers*_V1 + _V1 + i;
}

int CMRSRep::var_p(int i)
{
    return _V2 + _n_customers*_V1 + _V1 + _V1 + i;
}

int CMRSRep::var_s()
{
    return _V2 + _n_customers*_V1 + _V1 + _V1 + _V1;
}

int CMRSRep::var_t()
{
    return _V2 + _n_customers*_V1 + _V1 + _V1 + _V1 + 1;
}

int CMRSRep::var_s1()
{
    return _V2 + _n_customers*_V1 + _V1 + _V1 + _V1 + 2;
}

int CMRSRep::var_t1()
{
    return _V2 + _n_customers*_V1 + _V1 + _V1 + _V1 + 3;
}

v_index CMRSRep::index_x()
{
    return _index_x;
}

v_index CMRSRep::index_y()
{
    return _index_y;
}

v_index CMRSRep::index_z()
{
    return _index_z;
}

v_index CMRSRep::index_f()
{
    return _index_f;
}

v_index CMRSRep::index_p()
{
    return _index_p;
}

v_index CMRSRep::index_connection_cost()
{
    return _index_connection_cost;
}

v_index CMRSRep::index_routing_cost()
{
    return _index_routing_cost;
}

vector<vector<double> > CMRSRep::connection_cost()
{
    return _connection_cost;
}

vector<vector<double> > CMRSRep::routing_cost()
{
    return _routing_cost;
}

vector<string> CMRSRep::customers_labels()
{
    return _customers_labels;
}

vector<string> CMRSRep::steiners_labels()
{
    return _steiners_labels;
}

string CMRSRep::depot0_label()
{
    return _depot0_label;
}

string CMRSRep::depot1_label()
{
    return _depot1_label;
}

string CMRSRep::label_x(int index, int i)
{
    if (i >= 2 + _n_customers)
        return _steiners_labels[i - 2 - _n_customers];
    else if (i >= 2)
        return _customers_labels[i - 2];
    else if (i == 1)
        return _depot1_label;
    else if (i == 0)
        return _depot0_label;
}

string CMRSRep::label_y(int index, int i)
{
    if (i < _n_customers)
        return _customers_labels[i];
    else if ((index == 1) && (i >= _n_customers))
        return _steiners_labels[i - _n_customers];
}

