#include "SeparacionAnilloOEstrella2.h"
#include "CMRSRep.h"
#include "CMRSPException.h"
#include "util.h"

#include <vector>
#include <set>
#include <cmath>
using namespace std;


SeparacionAnilloOEstrella2::SeparacionAnilloOEstrella2(CMRSRep* cmrs_rep)
{
    this->cmrs_rep = cmrs_rep;
    _cortes_aplicados = 0;
}

string SeparacionAnilloOEstrella2::nombre_familia()
{
    return "anillo_o_estrella_2";
}

bool SeparacionAnilloOEstrella2::separar(CPXCENVptr env, double* x, void *cbdata, int wherefrom) throw(CMRSPException)
{    
    /**************************************************************************
     * 
     * 
     * 
     **************************************************************************/
    bool cuts_added = false;
    _cortes_aplicados = 0;
    
    int n_rings = cmrs_rep->n_rings();
    int ring_capacity = cmrs_rep->ring_capacity();
    int n_customers = cmrs_rep->n_customers();
    int n_steiners = cmrs_rep->n_steiners();
    v_index index_connection_cost = cmrs_rep->index_connection_cost();
    v_index index_routing_cost = cmrs_rep->index_routing_cost();
    v_index index_x = cmrs_rep->index_x();
    v_index index_y = cmrs_rep->index_y();
    v_index index_f = cmrs_rep->index_f();
    v_index index_z = cmrs_rep->index_z();
    vector<vector<double> > connection_cost = cmrs_rep->connection_cost();
    vector<vector<double> > routing_cost = cmrs_rep->routing_cost();
    int _V = cmrs_rep->V();
    int _V1 = cmrs_rep->V1();
    
    for (int u = 0; u < n_customers; u++)
    {
        for (int v1 = 0; v1 < _V1; v1++)
        {
            if (u == v1) continue;
            
            double s = x[cmrs_rep->var_x(index_x.CUSTOMER + v1, index_x.CUSTOMER + u)] + 
                       x[cmrs_rep->var_x(index_x.CUSTOMER + u, index_x.CUSTOMER + v1)] + 
                       x[cmrs_rep->var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v1)];
            for (int v = 0; v < _V1; v++)
            {
                if (v == v1) continue;
                
                s += x[cmrs_rep->var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v)];
            }
            
            if (s > 1.0 + m_eps)
            {
                int numnz = 2 + _V1;
                int index = 0;
                int rmatind[numnz];
                double rmatval[numnz];
                double rhs_value = 1.0;
                
                rmatind[0] = cmrs_rep->var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v1);
                rmatval[0] = 1.0;
                
                rmatind[1] = cmrs_rep->var_x(index_x.CUSTOMER + v1, index_x.CUSTOMER + u);
                rmatval[1] = 1.0;
                
                rmatind[2] = cmrs_rep->var_x(index_x.CUSTOMER + u, index_x.CUSTOMER + v1);
                rmatval[2] = 1.0;
                
                index = 3;
                for (int v = 0; v < _V1; v++)
                {
                    if (v == v1) continue;
                    
                    rmatind[index] = cmrs_rep->var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v);
                    rmatval[index] = 1.0;
                    index++;
                }
                
                cuts_added = true;
                int status = CPXcutcallbackadd (env, cbdata, wherefrom, numnz, rhs_value, 'L', rmatind, rmatval);
                if (status)
                {
                    char errmsg[__MSG_ERR_LEN];
                    CPXgeterrorstring (env, status, errmsg);
                    throw CMRSPException("Error agregando restricciones de familia anillo_o_estrella_2: "+string(errmsg));
                }
                _cortes_aplicados++;
            }
            
        }
    }
    
    return cuts_added;
}


int SeparacionAnilloOEstrella2::cortes_aplicados()
{
    return _cortes_aplicados;
}
