#include "SeparacionComponentes.h"
#include "CMRSRep.h"
#include "CMRSPException.h"
#include "util.h"

#include <vector>
#include <set>
#include <cmath>
#include <iostream>
using namespace std;


SeparacionComponentes::SeparacionComponentes(CMRSRep* cmrs_rep)
{
    this->cmrs_rep = cmrs_rep;
    _cortes_aplicados = 0;
}

string SeparacionComponentes::nombre_familia()
{
    return "componentes";
}

bool SeparacionComponentes::separar(CPXCENVptr env, double* x, void *cbdata, int wherefrom) throw(CMRSPException)
{    
    /**************************************************************************
     * Busqueda de las componentes conexas que contienen clientes
     * aislados del deposito
     **************************************************************************/
    _cortes_aplicados = 0;
    
    int n_rings = cmrs_rep->n_rings();
    int ring_capacity = cmrs_rep->ring_capacity();
    int n_customers = cmrs_rep->n_customers();
    int n_steiners = cmrs_rep->n_steiners();
    v_index index_connection_cost = cmrs_rep->index_connection_cost();
    v_index index_routing_cost = cmrs_rep->index_routing_cost();
    v_index index_x = cmrs_rep->index_x();
    v_index index_y = cmrs_rep->index_y();
    v_index index_f = cmrs_rep->index_f();
    v_index index_z = cmrs_rep->index_z();
    vector<vector<double> > connection_cost = cmrs_rep->connection_cost();
    vector<vector<double> > routing_cost = cmrs_rep->routing_cost();
    int _V = cmrs_rep->V();
    int _V1 = cmrs_rep->V1();
    
    int status;
    int solstat;
    double objval;
    bool cuts_generated = false;
    bool cuts_added = false;
    vector<vector<int> > components;
    vector<int> n_component_customers;
    vector<int> n_connected_to_component_customers;
    int index_x_customer = index_x.CUSTOMER;
    int index_y_customer = index_y.CUSTOMER;
    set<int> not_visited_customers;
    vector<bool> visited = vector<bool>(n_customers + n_steiners);
    int cust, n_cust;
    int x_source, x_target, y_source, y_target;
    
    for (int i = 0; i < n_customers; i++)
        not_visited_customers.insert(i);
    
    for (int i = 0; i < n_customers+n_steiners; i++)
        visited[i] = false;
    
    while (!not_visited_customers.empty())
    {
        vector<int> component = vector<int>();
        vector<int> visit_stack = vector<int>();
        cust = *not_visited_customers.begin();
        
        component.push_back(cust); n_cust = 1;
        not_visited_customers.erase(cust);
        visited[cust] = true;
        visit_stack.push_back(cust);
        while (visit_stack.size() > 0)
        {
            bool depot = false;
            
            int source = visit_stack.back();
            visit_stack.pop_back();
            
            x_source = index_x_customer + source;
            y_source = index_y_customer + source;
            for (int target = 0; target < _V1; target++)
            {
                x_target = index_x_customer + target;
                y_target = index_y_customer + target;
                
                if (!visited[target])
                {
                    if ( (x[cmrs_rep->var_x(x_source, index_x.D1)] > m_eps) || (x[cmrs_rep->var_x(index_x.D0, x_source)] > m_eps) )
                        depot = true;
                    
                    /* verifica si esta conectado por medio de una conexion anillo o estrella */
                    if ( (x[cmrs_rep->var_x(x_source, x_target)] > m_eps) || (x[cmrs_rep->var_x(x_target, x_source)] > m_eps) ||
                         ( (source < n_customers) && (x[cmrs_rep->var_y(y_source, y_target)] > m_eps) ) ||
                         ( (target < n_customers) && (x[cmrs_rep->var_y(y_target, y_source)] > m_eps) )
                       )
                    {
                        component.push_back(target);
                        visit_stack.push_back(target);
                        if (target < n_customers)
                        {
                            not_visited_customers.erase(target);
                            n_cust++;
                        }
                        visited[target] = true;
                    }                    
                }
            }
            
            if (!depot) cuts_generated = true;
            // if (depot) cout << "el componente contiene al deposito" << endl; else cout << "el componente NO contiene al deposito" << endl;
        }
        
        if (component.size() > 1)
        {
            components.push_back(component);
            n_component_customers.push_back(n_cust);
        }
    }    
    
    /**************************************************************************
     * Agregando restricciones a la formulación
     **************************************************************************/
    if (cuts_generated)
    {
        vector<int> in_component(n_customers + n_steiners);
        for (int i = 0; i < n_customers + n_steiners; i++)
            in_component[i] = -1;
        
        for (int k = 0; k < components.size(); k++)
        {
            for (int i = 0; i < components[k].size(); i++)
            {
                in_component[components[k][i]] = k;
            }
        }
        
        if (components.size() > 1)
        {
            int numnz = 2*_V*_V1 + _V;
            
            for (int k = 0; k < components.size(); k++)
            {
                if (components[k].size() > 1)
                {
                    int rmatind[numnz];
                    double rmatval[numnz];
                    int rmatbeg[1];
                    double rhs_value;
                    int index = 0;
                    double lhs_value = 0.0;
                    rmatbeg[0] = index;
int clientes = 0;
                    for (int i = 0; i < components[k].size(); i++)
                    {
//if (components[k][i] < n_customers) clientes++;
                        for (int j = 0; j < _V1; j++)
                        {
                            if (in_component[j] != k)
                            {
                                //cout << "x_"; if (components[k][i] < n_customers) cout << customers_labels[components[k][i]]; else cout << steiners_labels[components[k][i] - n_customers]; cout << "_"; if (j < n_customers) cout << customers_labels[j]; else cout << steiners_labels[j - n_customers];cout << " ";
                                lhs_value += x[cmrs_rep->var_x(index_x_customer + components[k][i], index_x_customer + j)];
                                rmatind[index] = cmrs_rep->var_x(index_x_customer + components[k][i], index_x_customer + j);
                                rmatval[index] = 1.0;
                                index++;
                                
                                if (components[k][i] < n_customers) // i es cliente
                                {
                                    //cout << "y_" << customers_labels[components[k][i]]; cout << "_"; if (j < n_customers) cout << customers_labels[j]; else cout << steiners_labels[j - n_customers];cout << " ";
                                    lhs_value += x[cmrs_rep->var_y(index_y_customer + components[k][i], index_y_customer + j)];
                                    rmatind[index] = cmrs_rep->var_y(index_y_customer + components[k][i], index_y_customer + j);
                                    rmatval[index] = 1.0;
                                    index++;
                                }
                            
                            }
                        }
                        
                        //cout << "x_"; if (components[k][i] < n_customers) cout << customers_labels[components[k][i]]; else cout << steiners_labels[components[k][i] - n_customers]; cout << "_D1";cout << " ";
                        lhs_value += x[cmrs_rep->var_x(index_x_customer + components[k][i], index_x.D1)];
                        rmatind[index] = cmrs_rep->var_x(index_x_customer + components[k][i], index_x.D1);
                        rmatval[index] = 1.0;
                        index++;
                        
                        if (i == components[k].size() - 1)
                        {
                            rhs_value = ceil( ((double)n_component_customers[k])/ring_capacity );
                            if (rhs_value - lhs_value > m_eps) 
                            {
//cout << "#Clientes =  " << clientes << " rhs_value = " << rhs_value << endl;

//                                cout << "la restriccion corta la solucion" << endl; 
                                // cout << "lhs = " << lhs_value << ", rhs = " << rhs_value << endl;
                                cuts_added = true;
                                _cortes_aplicados++;
                                //cuts_per_family[INDEX_CAPACITY]++;
                            }
                            else 
                            {
                                // cout << "la restriccion no corta la solucion" << endl;
                                continue;
                            }
                            // cout << ">=" << rhs << endl;
                            
                            // status = CPXaddrows(env, cmrsp_model, 0, 1 /*cindex*/, index, rhs, sense, rmatbeg, rmatind, rmatval, NULL, NULL /*rowname*/);
                            status = CPXcutcallbackadd (env, cbdata, wherefrom, index, rhs_value, 'G', rmatind, rmatval);
                            
                            if (status)
                            {
                                char errmsg[__MSG_ERR_LEN];
                                CPXgeterrorstring (env, status, errmsg);
                                throw CMRSPException("Error agregando restricciones de 1er familia: "+string(errmsg));
                            }
                            
//                             cout << "COMPONENTE: " << endl;
//                             vector<int> component = components[k];
//                             for (int i = 0; i < component.size(); i++)
//                             {
//                                 if (component[i] < n_customers)
//                                     cout << customers_labels[component[i]] << " ";
//                                 else
//                                     cout << steiners_labels[component[i]-n_customers] << " ";
//                             }
//                             cout << endl << "----------" << endl;
                        }
                        
                    }                    
                    
                }
            }
            
        }
    }
        
    return cuts_added;
}

int SeparacionComponentes::cortes_aplicados()
{
    return _cortes_aplicados;
}




// bool CMRSP::ring_xor_star_cuts1(double* x, void *cbdata, int wherefrom) throw(CMRSPException)
// {
//     bool cuts_applied = false;
//     
//     for (int u = 0; u < n_customers; u++)
//     {
//         for (int v = 0; v < _V1; v++)
//         {
//             if (u == v) continue;
//             
//             /******************************************************************
//              * f_{u} <= 1 + M(1 - y_{u v'}) con M = \vert U \vert
//              *                          \forall u \in U, \forall v' \in V'
//              ******************************************************************/
//             if (x[var_f(index_f.CUSTOMER + u)] + n_customers * x[var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v)] > 1 + n_customers + machine_eps )
//             {
//                 /*
//                 cout << "aplicando ring_xor_star_1 corte: " << tmp_colname[var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v)]
//                                                             << " = " << x[var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v)]
//                                                             << ", " << tmp_colname[var_f(index_f.CUSTOMER + u)]
//                                                             << " = " << x[var_f(index_f.CUSTOMER + u)] << endl;
//                 */
//                 int numnz = 2;
//                 int rmatind[numnz];
//                 double rmatval[numnz];
//                 double rhs_value = n_customers + 1;
//                 
//                 rmatind[0] = var_f(index_f.CUSTOMER + u);
//                 rmatval[0] = 1.0;
//                 
//                 rmatind[1] = var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v);
//                 rmatval[1] = n_customers;
//                 cuts_applied = true;
//                 
//                 cuts_per_family[INDEX_RING_XOR_STAR_1]++;
//                 int status = CPXcutcallbackadd (env, cbdata, wherefrom, numnz, rhs_value, 'L', rmatind, rmatval);
//                 if (status)
//                 {
//                     char errmsg[__MSG_ERR_LEN];
//                     CPXgeterrorstring (env, status, errmsg);
//                     throw CMRSPException("Error agregando restricciones de familia ring_xor_star_1: "+string(errmsg));
//                 }
//             }
//             
//             /******************************************************************
//              * x_{u v'} + x_{v' u} + y_{u v} \leq 1 \forall u \in U, 
//              *                                      \forall v, v' \in V'
//              ******************************************************************/
//             if (x[var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v)] > machine_eps)
//             {
//                 for (int v1 = 0; v1 < _V1; v1++)
//                 {
//                     if (v == v1 || v1 == u) continue;
//                     
//                     if (x[var_x(index_x.CUSTOMER + u, index_x.CUSTOMER + v1)]   + 
//                         x[var_x(index_x.CUSTOMER + v1, index_x.CUSTOMER + u)]   + 
//                         x[var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v)]    +
//                         x[var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v1)]   > 1.0 + machine_eps)
//                     {
//                         /*
//                         cout << "aplicando ring_xor_star_2 corte: " << tmp_colname[var_x(index_x.CUSTOMER + u, index_x.CUSTOMER + v1)]
//                                                                     << " = " << x[var_x(index_x.CUSTOMER + u, index_x.CUSTOMER + v1)]
//                                                                     << ", " << tmp_colname[var_x(index_x.CUSTOMER + v1, index_x.CUSTOMER + u)]
//                                                                     << " = " << x[var_x(index_x.CUSTOMER + v1, index_x.CUSTOMER + u)]
//                                                                     << ", " << tmp_colname[var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v)]
//                                                                     << " = " << x[var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v)]
//                                                                     << ", " << tmp_colname[var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v1)]
//                                                                     << " = " << x[var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v1)] << endl;
//                         */
//                         int numnz = 4;
//                         int index = 0;
//                         int rmatind[numnz];
//                         double rmatval[numnz];
//                         double rhs_value = 1.0;
//                         
//                         rmatind[0] = var_x(index_x.CUSTOMER + u, index_x.CUSTOMER + v1);
//                         rmatval[0] = 1.0;
//                         
//                         rmatind[1] = var_x(index_x.CUSTOMER + v1, index_x.CUSTOMER + u);
//                         rmatval[1] = 1.0;
//                         
//                         rmatind[2] = var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v);
//                         rmatval[2] = 1.0;
//                         
//                         rmatind[3] = var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v1);
//                         rmatval[3] = 1.0;
//                         
//                         cuts_applied = true;                        
//                         cuts_per_family[INDEX_RING_XOR_STAR_2]++;
//                         int status = CPXcutcallbackadd (env, cbdata, wherefrom, numnz, rhs_value, 'L', rmatind, rmatval);
//                         if (status)
//                         {
//                             char errmsg[__MSG_ERR_LEN];
//                             CPXgeterrorstring (env, status, errmsg);
//                             throw CMRSPException("Error agregando restricciones de familia ring_xor_star_2: "+string(errmsg));
//                         }
//                     }
//                     
//                     
//                 }
//             }
//         }
//     }
// 
//     return cuts_applied;
// }
// 
// bool CMRSP::ring_xor_star_cuts2(double* x, void *cbdata, int wherefrom) throw(CMRSPException)
// {
//     return false;
// }
// 
// bool CMRSP::separate_path(double* x, void *cbdata, int wherefrom) throw(CMRSPException)
// {
//     bool cuts_applied = false;
//     vector<bool> visited(_V);
//     
//     for (int i = 0; i < _V1; i++) visited[i] = false;
//     
//     for (int v = 0; v < _V1; v++)
//     {
//         if (visited[v]) continue;
//         
//         if (x[var_x(index_x.D0, index_x.CUSTOMER + v)] > machine_eps)
//         {
//             vector<int> path;
//             vector<int> not_visited;
//             int customer_count = 0;            
//             not_visited.push_back(v);
//             
//             while ( !not_visited.empty() )
//             {
//                 int current = not_visited.back();
//                 not_visited.pop_back();
//                 
//                 path.push_back(current);
//                 visited[current] = true;
//                 
//                 for (int v1 = 0; v1 < _V1; v1++)
//                 {
//                     if (visited[v1]) continue;
//                     
//                     if ( (x[var_x(index_x.CUSTOMER + current, index_x.CUSTOMER + v1)] > machine_eps || 
//                           x[var_x(index_x.CUSTOMER + v1, index_x.CUSTOMER + current)] > machine_eps )
//                          && find(not_visited.begin(), not_visited.end(), v1) == path.end()
//                        )
//                     {
//                         not_visited.push_back(v1);
//                         if (v1 < n_customers) customer_count++;
//                     }
//                 }            
//             }
//             
//             if (customer_count > ring_capacity)
//             {
//                 int numnz = (_V1 - path.size() + 1)*path.size();
//                 int index = 0, index1 = 0;
//                 int rmatind[numnz];
//                 double rmatval[numnz];
//                 int rmatind1[numnz];
//                 double rmatval1[numnz];
//                 double rhs_value = ceil(customer_count / ring_capacity);
//                 double lhs_value = 0.0, lhs_value1 = 0.0;
// // cout << "path.size() = " << path.size() << ", _V1 = " << _V1 << endl;
// // cout << "path = ";*/
// // for (int i = 0; i < path.size(); i++) cout << path[i] << " ";
// // cout << endl;
// // cout << "agregando camino" << endl;
//                 for (int i = 0; i < path.size(); i++)
//                 {
//                     for (int v = 0; v < _V1; v++)
//                     {
//                         if (find(path.begin(), path.end(), v) != path.end())
//                             continue;
//                         
// // cout << tmp_colname[var_x(index_x.CUSTOMER + path[i], index_x.CUSTOMER + v)] << endl;
//                         lhs_value += x[var_x(index_x.CUSTOMER + path[i], index_x.CUSTOMER + v)];
//                         rmatind[index] = var_x(index_x.CUSTOMER + path[i], index_x.CUSTOMER + v);
//                         rmatval[index] = 1.0;
//                         index++;
// // cout << "- index = " << index << ", numnz = " << numnz << endl;
//                         lhs_value1 += x[var_x(index_x.CUSTOMER + v, index_x.CUSTOMER + path[i])];
//                         rmatind1[index1] = var_x(index_x.CUSTOMER + v, index_x.CUSTOMER + path[i]);
//                         rmatval1[index1] = 1.0;
//                         index1++;
//                         
//                     }
//                     
//                     lhs_value += x[var_x(index_x.CUSTOMER + path[i], index_x.D1)];
//                     rmatind[index] = var_x(index_x.CUSTOMER + path[i], index_x.D1);
//                     rmatval[index] = 1.0;
//                     index++;
// 
//                     lhs_value1 += x[var_x(index_x.D0, index_x.CUSTOMER + path[i])];
//                     rmatind1[index1] = var_x(index_x.D0, index_x.CUSTOMER + path[i]);
//                     rmatval1[index1] = 1.0;
//                     index1++;
// // cout << "-- index = " << index << ", numnz = " << numnz << endl;
//                 }
//                                 
// /*cout << "rhs_value = " << rhs_value << endl;
// cout << "index = " << index << ", numnz = " << numnz << endl;
// cout << "--------------" << endl;*/
//                 
//                 if (lhs_value + machine_eps < rhs_value) 
//                 {
// // cout << "aplicando corte" << endl;
//                     cuts_applied = true;
//                     int status = CPXcutcallbackadd (env, cbdata, wherefrom, numnz, rhs_value, 'G', rmatind, rmatval);
//                     if (status)
//                     {
//                         char errmsg[__MSG_ERR_LEN];
//                         CPXgeterrorstring (env, status, errmsg);
//                         throw CMRSPException("Error agregando restricciones de familia caminos: "+string(errmsg));
//                     }
//                 }
//                 
//                 if (lhs_value1 + machine_eps < rhs_value) 
//                 {
// // cout << "aplicando corte" << endl;
//                     cuts_applied = true;
//                     int status = CPXcutcallbackadd (env, cbdata, wherefrom, numnz, rhs_value, 'G', rmatind1, rmatval1);
//                     if (status)
//                     {
//                         char errmsg[__MSG_ERR_LEN];
//                         CPXgeterrorstring (env, status, errmsg);
//                         throw CMRSPException("Error agregando restricciones de familia caminos: "+string(errmsg));
//                     }
//                 }
//                 
//             }
//         }
//     }
//     
//     return cuts_applied;    
// }
// 
// bool CMRSP::separate1(double* x, void *cbdata, int wherefrom) throw(CMRSPException)
// {    
//     /**************************************************************************
//      * Busqueda de las componentes conexas que contienen clientes
//      * aislados del deposito
//      **************************************************************************/
//     //if (cuts1b_applied > 2) return false;
// 
//     int status;
//     int solstat;
//     double objval;
//     bool cuts_generated = false;
//     bool cuts_added = false;
//     vector<vector<int> > components;
//     vector<int> n_component_customers;
//     int index_x_customer = index_x.CUSTOMER;
//     int index_y_customer = index_y.CUSTOMER;
//     set<int> not_visited_customers;
//     vector<bool> visited = vector<bool>(n_customers + n_steiners);
//     int cust, n_cust;
//     int x_source, x_target, y_source, y_target;
//     
//     for (int i = 0; i < n_customers; i++)
//         not_visited_customers.insert(i);
//     
//     for (int i = 0; i < n_customers+n_steiners; i++)
//         visited[i] = false;
//     
//     while (!not_visited_customers.empty())
//     {
//         vector<int> component = vector<int>();
//         vector<int> visit_stack = vector<int>();
//         cust = *not_visited_customers.begin();
//         
//         component.push_back(cust); n_cust = 1;
//         visit_stack.push_back(cust);
//         while (visit_stack.size() > 0)
//         {
//             bool depot = false;
//             
//             int source = visit_stack.back();
//             visit_stack.pop_back();
//             visited[source] = true;
//             not_visited_customers.erase(source);
//             
//             x_source = index_x_customer + source;
//             y_source = index_y_customer + source;
//             for (int target = 0; target < _V1; target++)
//             {
//                 x_target = index_x_customer + target;
//                 y_target = index_y_customer + target;
//                 
//                 if (!visited[target])
//                 {
//                     if (target < n_customers)
//                     {
//                         not_visited_customers.erase(target);
//                     }
//                     visited[target] = true;
//                     
//                     if ( (x[var_x(x_source, index_x.D1)] > machine_eps) || (x[var_x(index_x.D0, x_source)] > machine_eps) )
//                     {
//                         depot = true;
//                     }
//                     
//                     /* verifica si esta conectado por medio de una conexion anillo o estrella */
//                     if ( (x[var_x(x_source, x_target)] > machine_eps) || (x[var_x(x_target, x_source)] > machine_eps) ||
//                          ( (source < n_customers) && (x[var_y(y_source, y_target)] > machine_eps) ) ||
//                          ( (target < n_customers) && (x[var_y(y_target, y_source)] > machine_eps) )
//                        )
//                     {
//                         component.push_back(target);
//                         visit_stack.push_back(target);
//                         if (target < n_customers)
//                         {
//                             n_cust++;
//                         }
//                     }
//                 }
//             }
//             
//             if (!depot) cuts_generated = true;
//             // if (depot) cout << "el componente contiene al deposito" << endl; else cout << "el componente NO contiene al deposito" << endl;
//         }
//         
//         if (component.size() > 1)
//         {
//             components.push_back(component);
//             n_component_customers.push_back(n_cust);
//         }
//     }    
//     
//     /**************************************************************************
//      * Agregando restricciones a la formulación
//      **************************************************************************/
//     if (cuts_generated)
//     {
//         vector<int> in_component(n_customers + n_steiners);
//         for (int i = 0; i < n_customers + n_steiners; i++)
//             in_component[i] = -1;
//         
//         for (int k = 0; k < components.size(); k++)
//         {
//             for (int i = 0; i < components[k].size(); i++)
//             {
//                 in_component[components[k][i]] = k;
//             }
//         }
//         
//         if (components.size() > 1)
//         {
//             int numnz = 2*_V*_V1 + _V;
//             
//             for (int k = 0; k < components.size(); k++)
//             {
//                 if (components[k].size() > 1)
//                 {
//                     int rmatind[numnz];
//                     double rmatval[numnz];
//                     int rmatbeg[1];
//                     double rhs_value;
//                     int index = 0;
//                     double lhs_value = 0.0;
//                     rmatbeg[0] = index;
//                     for (int i = 0; i < components[k].size(); i++)
//                     {
//                         int visited_customers_count = 0;
//                         for (int j = 0; j < _V1; j++)
//                         {
//                             if (in_component[j] != k)
//                             {
//                                 //cout << "x_"; if (components[k][i] < n_customers) cout << customers_labels[components[k][i]]; else cout << steiners_labels[components[k][i] - n_customers]; cout << "_"; if (j < n_customers) cout << customers_labels[j]; else cout << steiners_labels[j - n_customers];cout << " ";
//                                 lhs_value += x[var_x(index_x_customer + j, index_x_customer + components[k][i])];
//                                 rmatind[index] = var_x(index_x_customer + j, index_x_customer + components[k][i]);
//                                 rmatval[index] = (ring_capacity - 1.0) / ring_capacity; // = 1.0 - 1.0/Q
//                                 index++;
//                                 
//                                 if (j < n_customers) // j es cliente
//                                 {
//                                     //cout << "y_" << customers_labels[components[k][i]]; cout << "_"; if (j < n_customers) cout << customers_labels[j]; else cout << steiners_labels[j - n_customers];cout << " ";
//                                     lhs_value += x[var_y(index_y_customer + j, index_y_customer + components[k][i])];
//                                     rmatind[index] = var_y(index_y_customer + j, index_y_customer + components[k][i]);
//                                     rmatval[index] = -1.0/ring_capacity;
//                                     index++;
//                                     visited_customers_count++;
//                                 }
//                             }
//                             else
//                             {
//                                 //cout << "x_"; if (components[k][i] < n_customers) cout << customers_labels[components[k][i]]; else cout << steiners_labels[components[k][i] - n_customers]; cout << "_"; if (j < n_customers) cout << customers_labels[j]; else cout << steiners_labels[j - n_customers];cout << " ";
//                                 lhs_value += x[var_x(index_x_customer + j, index_x_customer + components[k][i])];
//                                 rmatind[index] = var_x(index_x_customer + j, index_x_customer + components[k][i]);
//                                 rmatval[index] = -1.0 / ring_capacity;
//                                 index++;
//                                 
//                                 if (j < n_customers) // i es cliente
//                                 {
//                                     //cout << "y_" << customers_labels[components[k][i]]; cout << "_"; if (j < n_customers) cout << customers_labels[j]; else cout << steiners_labels[j - n_customers];cout << " ";
//                                     lhs_value += x[var_y(index_y_customer + j, index_y_customer + components[k][i])];
//                                     rmatind[index] = var_y(index_y_customer + j, index_y_customer + components[k][i]);
//                                     rmatval[index] = -1.0 / ring_capacity;
//                                     index++;
//                                     visited_customers_count++;
//                                 }
//                             }
//                         }
//                         
//                         //cout << "x_"; if (components[k][i] < n_customers) cout << customers_labels[components[k][i]]; else cout << steiners_labels[components[k][i] - n_customers]; cout << "_D1";cout << " ";
//                         lhs_value += x[var_x(index_x.D0, index_x_customer + components[k][i])];
//                         rmatind[index] = var_x(index_x.D0, index_x_customer + components[k][i]);
//                         rmatval[index] = (ring_capacity - 1.0)/ring_capacity;
//                         index++;
//                         
//                         if (i == components[k].size() - 1)
//                         {
//                             // cout << "lhs = " << lhs_value << ", rhs = " << rhs << endl;
//                             rhs_value = 0.0;
// 
// /*                            if (lhs_value > machine_eps && visited_customers_count >= ring_capacity) 
//                             {
//                                 // cout << "la restriccion corta la solucion" << endl; 
//                                 cuts_added = true;
//                                 cuts1b_applied++;
//                             }
//                             else 
//                             {
//                                 // cout << "la restriccion no corta la solucion" << endl;
//                                 continue;
//                             }*/
//                             // cout << ">=" << rhs << endl;
//                             
//                             // cout << "agregando corte..." << endl;
//                             // status = CPXaddrows(env, cmrsp_model, 0, 1 /*cindex*/, index, rhs, sense, rmatbeg, rmatind, rmatval, NULL, NULL /*rowname*/);
// 
// 
// 
// cout << "COMPONENTE: " << endl;
// vector<int> component = components[k];
// for (int i = 0; i < component.size(); i++)
// {
//     if (component[i] < n_customers)
//         cout << customers_labels[component[i]] << " ";
//     else
//         cout << steiners_labels[component[i]-n_customers] << " ";
// }
// for (int t = 0; t < index; t++)
// {
//     cout << tmp_colname[rmatind[t]] << " = " << rmatval[t] << endl;
// }
// cout << "-------------------------------" << endl;
// cout << "agregando corte " << cuts1b_applied << endl;
//                             status = CPXcutcallbackadd (env, cbdata, wherefrom, index, rhs_value, 'G', rmatind, rmatval);
//                             
//                             if (status)
//                             {
//                                 char errmsg[__MSG_ERR_LEN];
//                                 CPXgeterrorstring (env, status, errmsg);
//                                 throw CMRSPException("Error agregando restricciones de 1er familia: "+string(errmsg));
//                             }
//                             
// //                             cout << "COMPONENTE: " << endl;
// //                             vector<int> component = components[k];
// //                             for (int i = 0; i < component.size(); i++)
// //                             {
// //                                 if (component[i] < n_customers)
// //                                     cout << customers_labels[component[i]] << " ";
// //                                 else
// //                                     cout << steiners_labels[component[i]-n_customers] << " ";
// //                             }
// //                             cout << endl << "----------" << endl;
//                         }
//                         
//                     }                    
//                     
//                 }
//             }
//             
//         }
//     }
//         
//     return cuts_added;
// }
// 
// bool CMRSP::apply_cuts1(double* x, void *cbdata, int wherefrom) throw(CMRSPException)
// {    
//     /**************************************************************************
//      * Busqueda de las componentes conexas que contienen clientes
//      * aislados del deposito
//      **************************************************************************/
//     int status;
//     int solstat;
//     double objval;
//     bool cuts_generated = false;
//     bool cuts_added = false;
//     vector<vector<int> > components;
//     vector<int> n_component_customers;
//     int index_x_customer = index_x.CUSTOMER;
//     int index_y_customer = index_y.CUSTOMER;
//     set<int> not_visited_customers;
//     vector<bool> visited = vector<bool>(n_customers + n_steiners);
//     int cust, n_cust;
//     int x_source, x_target, y_source, y_target;
//     
//     for (int i = 0; i < n_customers; i++)
//         not_visited_customers.insert(i);
//     
//     for (int i = 0; i < n_customers+n_steiners; i++)
//         visited[i] = false;
//     
//     while (!not_visited_customers.empty())
//     {
//         vector<int> component = vector<int>();
//         vector<int> visit_stack = vector<int>();
//         cust = *not_visited_customers.begin();
//         
//         component.push_back(cust); n_cust = 1;
//         not_visited_customers.erase(cust);
//         visited[cust] = true;
//         visit_stack.push_back(cust);
//         while (visit_stack.size() > 0)
//         {
//             bool depot = false;
//             
//             int source = visit_stack.back();
//             visit_stack.pop_back();
//             
//             x_source = index_x_customer + source;
//             y_source = index_y_customer + source;
//             for (int target = 0; target < _V1; target++)
//             {
//                 x_target = index_x_customer + target;
//                 y_target = index_y_customer + target;
//                 
//                 if (!visited[target])
//                 {
//                     if ( (x[var_x(x_source, index_x.D1)] > machine_eps) || (x[var_x(index_x.D0, x_source)] > machine_eps) )
//                         depot = true;
//                     
//                     /* verifica si esta conectado por medio de una conexion anillo o estrella */
//                     if ( (x[var_x(x_source, x_target)] > machine_eps) || (x[var_x(x_target, x_source)] > machine_eps) ||
//                          ( (source < n_customers) && (x[var_y(y_source, y_target)] > machine_eps) ) ||
//                          ( (target < n_customers) && (x[var_y(y_target, y_source)] > machine_eps) )
//                        )
//                     {
//                         component.push_back(target);
//                         visit_stack.push_back(target);
//                         if (target < n_customers)
//                         {
//                             not_visited_customers.erase(target);
//                             n_cust++;
//                         }
//                         visited[target] = true;
//                     }                    
//                 }
//             }
//             
//             if (!depot) cuts_generated = true;
//             // if (depot) cout << "el componente contiene al deposito" << endl; else cout << "el componente NO contiene al deposito" << endl;
//         }
//         
//         if (component.size() > 1)
//         {
//             components.push_back(component);
//             n_component_customers.push_back(n_cust);
//         }
//     }    
//     
//     /**************************************************************************
//      * Agregando restricciones a la formulación
//      **************************************************************************/
//     if (cuts_generated)
//     {
//         vector<int> in_component(n_customers + n_steiners);
//         for (int i = 0; i < n_customers + n_steiners; i++)
//             in_component[i] = -1;
//         
//         for (int k = 0; k < components.size(); k++)
//         {
//             for (int i = 0; i < components[k].size(); i++)
//             {
//                 in_component[components[k][i]] = k;
//             }
//         }
//         
//         if (components.size() > 1)
//         {
//             int numnz = 2*_V*_V1 + _V;
//             
//             for (int k = 0; k < components.size(); k++)
//             {
//                 if (components[k].size() > 1)
//                 {
//                     int rmatind[numnz];
//                     double rmatval[numnz];
//                     int rmatbeg[1];
//                     double rhs_value;
//                     int index = 0;
//                     double lhs_value = 0.0;
//                     rmatbeg[0] = index;
//                     for (int i = 0; i < components[k].size(); i++)
//                     {
//                         for (int j = 0; j < _V1; j++)
//                         {
//                             if (in_component[j] != k)
//                             {
//                                 //cout << "x_"; if (components[k][i] < n_customers) cout << customers_labels[components[k][i]]; else cout << steiners_labels[components[k][i] - n_customers]; cout << "_"; if (j < n_customers) cout << customers_labels[j]; else cout << steiners_labels[j - n_customers];cout << " ";
//                                 lhs_value += x[var_x(index_x_customer + components[k][i], index_x_customer + j)];
//                                 rmatind[index] = var_x(index_x_customer + components[k][i], index_x_customer + j);
//                                 rmatval[index] = 1.0;
//                                 index++;
//                                 
//                                 if (components[k][i] < n_customers) // i es cliente
//                                 {
//                                     //cout << "y_" << customers_labels[components[k][i]]; cout << "_"; if (j < n_customers) cout << customers_labels[j]; else cout << steiners_labels[j - n_customers];cout << " ";
//                                     lhs_value += x[var_y(index_y_customer + components[k][i], index_y_customer + j)];
//                                     rmatind[index] = var_y(index_y_customer + components[k][i], index_y_customer + j);
//                                     rmatval[index] = 1.0;
//                                     index++;
//                                 }
//                             }
//                         }
//                         
//                         //cout << "x_"; if (components[k][i] < n_customers) cout << customers_labels[components[k][i]]; else cout << steiners_labels[components[k][i] - n_customers]; cout << "_D1";cout << " ";
//                         lhs_value += x[var_x(index_x_customer + components[k][i], index_x.D1)];
//                         rmatind[index] = var_x(index_x_customer + components[k][i], index_x.D1);
//                         rmatval[index] = 1.0;
//                         index++;
//                         
//                         if (i == components[k].size() - 1)
//                         {
//                             rhs_value = ceil( ((double)n_component_customers[k])/ring_capacity );
//                             if (rhs_value - lhs_value > machine_eps) 
//                             {
//                                 // cout << "la restriccion corta la solucion" << endl; 
//                                 // cout << "lhs = " << lhs_value << ", rhs = " << rhs_value << endl;
//                                 cuts_added = true;
//                                 cuts1a_applied++;
//                                 cuts_per_family[INDEX_CAPACITY]++;
//                             }
//                             else 
//                             {
//                                 // cout << "la restriccion no corta la solucion" << endl;
//                                 continue;
//                             }
//                             // cout << ">=" << rhs << endl;
//                             
//                             // status = CPXaddrows(env, cmrsp_model, 0, 1 /*cindex*/, index, rhs, sense, rmatbeg, rmatind, rmatval, NULL, NULL /*rowname*/);
//                             status = CPXcutcallbackadd (env, cbdata, wherefrom, index, rhs_value, 'G', rmatind, rmatval);
//                             
//                             if (status)
//                             {
//                                 char errmsg[__MSG_ERR_LEN];
//                                 CPXgeterrorstring (env, status, errmsg);
//                                 throw CMRSPException("Error agregando restricciones de 1er familia: "+string(errmsg));
//                             }
//                             
// //                             cout << "COMPONENTE: " << endl;
// //                             vector<int> component = components[k];
// //                             for (int i = 0; i < component.size(); i++)
// //                             {
// //                                 if (component[i] < n_customers)
// //                                     cout << customers_labels[component[i]] << " ";
// //                                 else
// //                                     cout << steiners_labels[component[i]-n_customers] << " ";
// //                             }
// //                             cout << endl << "----------" << endl;
//                         }
//                         
//                     }                    
//                     
//                 }
//             }
//             
//         }
//     }
//         
//     return cuts_added;
// }
// 
// 
// bool CMRSP::apply_cuts4(double* x, void *cbdata, int wherefrom) throw(CMRSPException)
// {    
//     /**************************************************************************
//      * Busqueda de las componentes conexas que contienen clientes
//      * aislados del deposito
//      **************************************************************************/
//     int status;
//     int solstat;
//     double objval;
//     bool cuts_generated = false;
//     bool cuts_added = false;
//     vector<vector<int> > components;
//     vector<int> n_component_customers;
//     int index_x_customer = index_x.CUSTOMER;
//     int index_y_customer = index_y.CUSTOMER;
//     set<int> not_visited_customers;
//     vector<bool> visited = vector<bool>(n_customers + n_steiners);
//     int cust, n_cust;
//     int x_source, x_target, y_source, y_target;
//     
//     for (int i = 0; i < n_customers; i++)
//         not_visited_customers.insert(i);
//     
//     for (int i = 0; i < n_customers+n_steiners; i++)
//         visited[i] = false;
//     
//     while (!not_visited_customers.empty())
//     {
//         vector<int> component = vector<int>();
//         vector<int> visit_stack = vector<int>();
//         cust = *not_visited_customers.begin();
//         
//         component.push_back(cust); n_cust = 1;
//         not_visited_customers.erase(cust);
//         visited[cust] = true;
//         visit_stack.push_back(cust);
//         while (visit_stack.size() > 0)
//         {
//             bool depot = false;
//             
//             int source = visit_stack.back();
//             visit_stack.pop_back();
//             
//             x_source = index_x_customer + source;
//             y_source = index_y_customer + source;
//             for (int target = 0; target < _V1; target++)
//             {
//                 x_target = index_x_customer + target;
//                 y_target = index_y_customer + target;
//                 
//                 if (!visited[target])
//                 {
//                     if ( (x[var_x(x_source, index_x.D1)] > machine_eps) || (x[var_x(index_x.D0, x_source)] > machine_eps) )
//                         depot = true;
//                     
//                     /* verifica si esta conectado por medio de una conexion anillo o estrella */
//                     if ( (x[var_x(x_source, x_target)] > machine_eps) || (x[var_x(x_target, x_source)] > machine_eps) ||
//                          ( (source < n_customers) && (x[var_y(y_source, y_target)] > machine_eps) ) ||
//                          ( (target < n_customers) && (x[var_y(y_target, y_source)] > machine_eps) )
//                        )
//                     {
//                         component.push_back(target);
//                         visit_stack.push_back(target);
//                         if (target < n_customers)
//                         {
//                             not_visited_customers.erase(target);
//                             n_cust++;
//                         }
//                         visited[target] = true;
//                     }                    
//                 }
//             }
//             
//             if (!depot) cuts_generated = true;
//             // if (depot) cout << "el componente contiene al deposito" << endl; else cout << "el componente NO contiene al deposito" << endl;
//         }
//         
//         if (component.size() > 1)
//         {
//             components.push_back(component);
//             n_component_customers.push_back(n_cust);
//         }
//     }    
//     
//     /**************************************************************************
//      * Agregando restricciones a la formulación
//      **************************************************************************/
//     if (cuts_generated)
//     {
//         vector<int> in_component(n_customers + n_steiners);
//         for (int i = 0; i < n_customers + n_steiners; i++)
//             in_component[i] = -1;
//         
//         for (int k = 0; k < components.size(); k++)
//         {
//             for (int i = 0; i < components[k].size(); i++)
//             {
//                 in_component[components[k][i]] = k;
//             }
//         }
//         
//         if (components.size() > 1)
//         {
//             int numnz = 2*_V*_V1 + _V;
//             
//             for (int k = 0; k < components.size(); k++)
//             {
//                 if (components[k].size() > 1)
//                 {
//                     int rmatind[numnz];
//                     double rmatval[numnz];
//                     int rmatbeg[1];
//                     double rhs_value;
//                     int index = 0;
//                     double lhs_value = 0.0;
//                     rmatbeg[0] = index;
//                     for (int i = 0; i < components[k].size(); i++)
//                     {
//                         for (int j = 0; j < _V1; j++)
//                         {
//                             if (in_component[j] != k)
//                             {
//                                 //cout << "x_"; if (components[k][i] < n_customers) cout << customers_labels[components[k][i]]; else cout << steiners_labels[components[k][i] - n_customers]; cout << "_"; if (j < n_customers) cout << customers_labels[j]; else cout << steiners_labels[j - n_customers];cout << " ";
//                                 lhs_value += x[var_x(index_x_customer + components[k][i], index_x_customer + j)];
//                                 rmatind[index] = var_x(index_x_customer + components[k][i], index_x_customer + j);
//                                 rmatval[index] = 1.0;
//                                 index++;                            
//                             }
//                         }
//                         
//                         //cout << "x_"; if (components[k][i] < n_customers) cout << customers_labels[components[k][i]]; else cout << steiners_labels[components[k][i] - n_customers]; cout << "_D1";cout << " ";
//                         lhs_value += x[var_x(index_x_customer + components[k][i], index_x.D1)];
//                         rmatind[index] = var_x(index_x_customer + components[k][i], index_x.D1);
//                         rmatval[index] = 1.0;
//                         index++;
//                         
//                         if (i == components[k].size() - 1)
//                         {
//                             // cout << "lhs = " << lhs_value << ", rhs = " << rhs << endl;
//                             rhs_value = ceil( ((double)n_component_customers[k])/ring_capacity );
//                             if (rhs_value - lhs_value > machine_eps) 
//                             {
//                                 // cout << "la restriccion corta la solucion" << endl; 
//                                 cuts_added = true;
//                                 cuts4_applied++;
//                             }
//                             else 
//                             {
//                                 // cout << "la restriccion no corta la solucion" << endl;
//                                 continue;
//                             }
//                             // cout << ">=" << rhs << endl;
//                             
//                             // status = CPXaddrows(env, cmrsp_model, 0, 1 /*cindex*/, index, rhs, sense, rmatbeg, rmatind, rmatval, NULL, NULL /*rowname*/);
//                             status = CPXcutcallbackadd (env, cbdata, wherefrom, index, rhs_value, 'G', rmatind, rmatval);
//                             
//                             if (status)
//                             {
//                                 char errmsg[__MSG_ERR_LEN];
//                                 CPXgeterrorstring (env, status, errmsg);
//                                 throw CMRSPException("Error agregando restricciones de 1er familia: "+string(errmsg));
//                             }
//                             
// //                             cout << "COMPONENTE: " << endl;
// //                             vector<int> component = components[k];
// //                             for (int i = 0; i < component.size(); i++)
// //                             {
// //                                 if (component[i] < n_customers)
// //                                     cout << customers_labels[component[i]] << " ";
// //                                 else
// //                                     cout << steiners_labels[component[i]-n_customers] << " ";
// //                             }
// //                             cout << endl << "----------" << endl;
//                         }
//                         
//                     }                    
//                     
//                 }
//             }
//             
//         }
//     }
//         
//     return cuts_added;
// }
// 
// 
// // bool CMRSP::apply_cuts2(double* x, void *cbdata, int wherefrom) throw(CMRSPException)
// // {    
// //     /**************************************************************************
// //      * Busqueda de ciclos compuestos por clientes y steiners
// //      **************************************************************************/
// //     bool cuts_added = false;
// //     vector<vector<int> > circuits;
// //     int index_x_customer = index_x.CUSTOMER;
// //     int index_y_customer = index_y.CUSTOMER;
// //     set<int> not_visited;
// //     vector<int> visited(_V1);
// //     vector<int> parent(_V1);
// //     
// //     for (int i = 0; i < _V1; i++)
// //     {
// //         not_visited.insert(i);
// //         visited[i] = 0;
// //         parent[i] = -1;
// //     }
// //     
// //     while (!not_visited.empty())
// //     {
// //         vector<int> visit_stack;
// //         
// //         int current_node = *not_visited.begin();
// //         visited[current_node] = -1;
// //         not_visited.erase(current_node);
// //         
// //         bool circuit_detected = false;
// //         visit_stack.push_back(current_node);
// //         while (visit_stack.size() > 0)
// //         {
// //             current_node = visit_stack.back();
// //             visited[current_node] = -1;
// //             not_visited.erase(current_node);
// //             visit_stack.pop_back();
// //             
// //             int x_current_node = index_x_customer + current_node;
// //             int y_current_node = index_y_customer + current_node;
// //             bool adjacents_found = false;
// //             for (int adjacent_node = 0; adjacent_node < _V1; adjacent_node++)
// //             {
// //                 if ((parent[current_node] != adjacent_node) && (visited[adjacent_node] != 1))
// //                 {
// //                     int x_adjacent_node = index_x_customer + adjacent_node;
// //                     int y_adjacent_node = index_y_customer + adjacent_node;
// //                     if ( x[var_x(x_current_node, x_adjacent_node)] > machine_eps ||
// //                          x[var_x(x_adjacent_node, x_current_node)] > machine_eps ||
// //                          ( (y_current_node < n_customers) && (x[var_y(y_current_node, y_adjacent_node)] > machine_eps) ) ||
// //                          ( (y_adjacent_node < n_customers) && (x[var_y(y_adjacent_node, y_current_node)] > machine_eps) )
// //                     )
// //                     {
// //                         adjacents_found = true;
// //                         visit_stack.push_back(adjacent_node);
// //                         parent[adjacent_node] = current_node;
// //                         if ((parent[current_node] != adjacent_node) && (visited[adjacent_node] == -1))
// //                         {
// //                             if (adjacent_node < n_customers) cout << customers_labels[adjacent_node] << " ";
// //                             else cout << steiners_labels[adjacent_node - n_customers] << " ";
// //                             circuit_detected = true;
// //                             break;
// //                         }
// //                     }
// //                     if (circuit_detected) break;
// //                 }
// //             }
// //             
// //             if (circuit_detected) break;            
// //         }
// //         
// //         for (int i = 0; i < _V1; i++) if (visited[i] == -1) visited[i] = 1;
// //         
// //         vector<int> circuit;
// //         if (circuit_detected)
// //         {
// //             int first_node = visit_stack.back();
// //             int current_node = first_node;
// //             int adjacent_node = parent[current_node];
// //             circuit.push_back(current_node);
// //             while (adjacent_node != first_node)
// //             {
// //                 circuit.push_back(adjacent_node);
// //                 adjacent_node = parent[adjacent_node];
// //             }
// //             
// //             circuits.push_back(circuit);
// //         }
// //         
// //         
// // //         if (circuit_detected)
// // //         {        
// // //             cout << "CIRCUITO: " << endl;
// // //             for (int i = 0; i < circuit.size(); i++)
// // //             {
// // //                 if (circuit[i] < n_customers)
// // //                     cout << customers_labels[circuit[i]] << " ";
// // //                 else
// // //                     cout << steiners_labels[circuit[i] - n_customers] << " ";
// // //             }
// // //             cout << endl;
// // //         }
// //     }
// //     
// //     /**************************************************************************
// //      * Agregando restricciones a la formulación
// //      **************************************************************************/
// //     cout << "Lista de candidatos: " << endl;
// //     for (int c = 0; c < circuits.size(); c++)
// //     {
// //         vector<int> circuit = circuits[c];
// //         for (int i = 0; i < circuit.size(); i++)
// //         {
// //             if (circuit[i] < n_customers)
// //                 cout << customers_labels[circuit[i]] << " ";
// //             else
// //                 cout << steiners_labels[circuit[i] - n_customers] << " ";
// //         }
// //         cout << endl;
// //     }
// //     
// //     if (circuits.size() > 0)
// //     {        
// //         int numnz = 8*_V*_V*circuits.size();
// //         int rmatbeg[0];
// //         int rmatind[numnz];
// //         double rmatval[numnz];
// //         char sense[0];
// //         
// //         for (int i = 0; i < circuits.size(); i++)
// //         {
// //             int index = 0;
// //             double rhs_value = 0.0;
// //             double lhs_value;
// //             
// //             cout << "evaluando circuito " << i << endl;
// //             rmatbeg[0] = index;
// //             lhs_value = 0.0;
// //             for (int j = 0; j < circuits[i].size() - 1; j++)
// //             {
// //                 lhs_value += x[var_x(index_x_customer + circuits[i][j], index_x_customer + circuits[i][j+1])];
// //                 rmatind[index] = var_x(index_x_customer + circuits[i][j], index_x_customer + circuits[i][j+1]);
// //                 rmatval[index] = 1.0;
// //                 index++;
// //                 
// //                 lhs_value += x[var_x(index_x_customer + circuits[i][j+1], index_x_customer + circuits[i][j])];
// //                 rmatind[index] = var_x(index_x_customer + circuits[i][j+1], index_x_customer + circuits[i][j]);
// //                 rmatval[index] = 1.0;
// //                 index++;
// //                 
// //                 if (circuits[i][j] < n_customers)
// //                 {
// //                     lhs_value += x[var_y(index_y_customer + circuits[i][j], index_y_customer + circuits[i][j+1])];
// //                     rmatind[index] = var_y(index_y_customer + circuits[i][j], index_y_customer + circuits[i][j+1]);
// //                     rmatval[index] = 1.0;
// //                     index++;
// //                 }
// //                 
// //                 if (circuits[i][j+1] < n_customers)
// //                 {
// //                     lhs_value += x[var_y(index_y_customer + circuits[i][j+1], index_y_customer + circuits[i][j])];
// //                     rmatind[index] = var_y(index_y_customer + circuits[i][j+1], index_y_customer + circuits[i][j]);
// //                     rmatval[index] = 1.0;
// //                     index++;
// //                 }
// //             }
// //             
// //             lhs_value += x[var_x(index_x_customer + circuits[i][circuits[i].size()-1], index_x_customer + circuits[i][0])];
// //             rmatind[index] = var_x(index_x_customer + circuits[i][circuits[i].size()-1], index_x_customer + circuits[i][0]);
// //             rmatval[index] = 1.0;
// //             index++;
// //             
// //             lhs_value += x[var_x(index_x_customer + circuits[i][0], index_x_customer + circuits[i][circuits[i].size()-1])];
// //             rmatind[index] = var_x(index_x_customer + circuits[i][0], index_x_customer + circuits[i][circuits[i].size()-1]);
// //             rmatval[index] = 1.0;
// //             index++;
// //             
// //             if (circuits[i][circuits[i].size()-1] < n_customers)
// //             {
// //                 lhs_value += x[var_y(index_y_customer + circuits[i][circuits[i].size()-1], index_y_customer + circuits[i][0])];
// //                 rmatind[index] = var_y(index_y_customer + circuits[i][circuits[i].size()-1], index_y_customer + circuits[i][0]);
// //                 rmatval[index] = 1.0;
// //                 index++;
// //             }
// //             
// //             if (circuits[i][0] < n_customers)
// //             {
// //                 lhs_value += x[var_y(index_y_customer + circuits[i][0], index_y_customer + circuits[i][circuits[i].size()-1])];
// //                 rmatind[index] = var_y(index_y_customer + circuits[i][0], index_y_customer + circuits[i][circuits[i].size()-1]);
// //                 rmatval[index] = 1.0;
// //                 index++;
// //             }
// //             
// //             rhs_value = circuits[i].size() - 1;
// //             cout << "lhs_value = " << lhs_value << endl;
// //             if (lhs_value - rhs_value > machine_eps) cout << "la restriccion corta la solucion" << endl; else cout << "la restriccion no corta la solucion" << endl;
// //             if (lhs_value - rhs_value > machine_eps)
// //             {
// //                 cuts_added = true;
// //                 cuts2_applied++;
// //                 // status = CPXaddrows(env, cmrsp_model, 0, 1 /*cindex*/, index, rhs, sense, rmatbeg, rmatind, rmatval, NULL, NULL /*rowname*/);
// //                 status = CPXcutcallbackadd (env, cbdata, wherefrom, index, rhs_value, 'L', rmatind, rmatval);
// //                 
// //                 if (status)
// //                 {
// //                     char errmsg[__MSG_ERR_LEN];
// //                     CPXgeterrorstring (env, status, errmsg);
// //                     throw CMRSPException("Error agregando restricciones de 2da familia: "+string(errmsg));
// //                 }
// //             }
// //         }
// // 
// //     }
// //     
// //     return cuts_added;
// // }
// 
// 
// // bool CMRSP::apply_cuts3() throw(CMRSPException)
// // {
// //     /**************************************************************************
// //      * Obtiene la solucion encontrada por CPLEX
// //      **************************************************************************/
// //     bool cuts_added = false;
// //     int cur_numrows = CPXgetnumrows(env, cmrsp_model);
// //     int cur_numcols = CPXgetnumcols(env, cmrsp_model);
// //     
// //     int      solstat;
// //     double   objval;
// //     double*  x = (double *) malloc (cur_numcols * sizeof(double));
// //     
// //     int status = CPXgetx(env, cmrsp_model, x, 0, cur_numcols-1);
// //     if (status)
// //     {
// //         cerr << "Error obteniendo la solucion" << endl;
// //         char errmsg[__MSG_ERR_LEN];
// //         CPXgeterrorstring (env, status, errmsg);
// //         cerr << errmsg << endl;
// //         CPXcloseCPLEX(&env);
// //         free(x);
// //         throw CMRSPException(errmsg);
// //     }
// //     
// //     /**************************************************************************
// //      * Busqueda de estructuras de arcos anillo y arcos estrella de la forma
// //      * (u, v), (u', v), (u, u') con u y u' nodos cliente 
// //      * y v nodos cliente o steiner
// //      **************************************************************************/
// //     vector<vector<int> > candidates;
// //     int index_x_customer = index_x.CUSTOMER;
// //     int index_y_customer = index_y.CUSTOMER;
// //     int x_customer_i, x_customer_j, y_customer_i, y_customer_j, y_node_k;
// //     
// //     for (int i = 0; i < n_customers; i++)
// //     {
// //         x_customer_i = index_x_customer + i;
// //         y_customer_i = index_y_customer + i;
// //         
// //         for (int j = 0; j < i; j++)
// //         {
// //             x_customer_j = index_x_customer + j;
// //             y_customer_j = index_y_customer + j;
// //             
// //             for (int k = 0; k < _V1; k++)
// //             {
// //                 if ((k == i) || (k == j)) continue;
// //                 
// //                 y_node_k = index_y_customer + k;
// //                 
// //                 if (    ( x[var_y(y_customer_i, y_node_k)] > machine_eps ) &&
// //                         ( x[var_y(y_customer_j, y_node_k)] > machine_eps ) &&
// //                         ( (x[var_x(x_customer_i, x_customer_j)] > machine_eps ) ||  
// //                           (x[var_x(x_customer_j, x_customer_i)] > machine_eps )    )    )
// //                 {
// //                     vector<int> candidate = vector<int>(3);
// //                     candidate[0] = i; candidate[1] = j; candidate[2] = k;
// //                     candidates.push_back(candidate);
// //                 }
// // 
// //             }
// //             
// //         }
// //         
// //     }
// //     
// //     /**************************************************************************
// //      * Aplicacion de cortes
// //      **************************************************************************/
// //     cout << "Lista de candidatos: " << endl;
// //     for (int i = 0; i < candidates.size(); i++)
// //     {
// //         for (int j = 0; j < 3; j++)
// //         {
// //             if (candidates[i][j] < n_customers) cout << customers_labels[candidates[i][j]] << " ";
// //             else cout << steiners_labels[candidates[i][j] - n_customers] << " ";
// //             
// //         }
// //         cout << endl;
// //     }
// //     
// //     
// //     int numnz = 2*(_V1 + 1) + 1;
// //     int rmatbeg[1];
// //     int rmatind[numnz];
// //     double rmatval[numnz];
// //     double rhs[1];
// //     char sense[1];
// //     int index = 0, cindex = 0;
// //     
// //     double lhs_value;
// //     int x_node_k, y_node_j;
// //     
// //     for (int i = 0; i < candidates.size(); i++)
// //     {        
// //         // analizar los casos:
// //         // Debe ser: 1/2 suma(v'=0..V1+{d0}) x[v'][u] + suma(v'=0..V1+{d1}) x[u][v'] <= 1 - y[u][v]
// //         // (j == 0) (caso 1) u = candidates[i][0] v = candidates[i][2] 
// //         // (j == 1) (caso 2) u = candidates[i][1] v = candidates[i][2]
// //         // lhs = (1/2 suma(v'=0..V1+{d0}) x[v'][u] + suma(v'=0..V1+{d1}) x[u][v']) + y[u][v]
// //         // rhs = 1
// //         for (int j = 0; j <= 1; j++)
// //         {
// //             lhs_value = 0.0;
// //             index = 0;
// //             
// //             x_customer_j = index_x_customer + candidates[i][j];
// //             y_customer_j = index_y_customer + candidates[i][j];
// //             y_node_j = candidates[i][2];
// //             
// //             for (int k = 0; k < _V1; k++)
// //             {
// //                 x_node_k = index_x_customer + k;
// //                 
// //                 if (x[var_x(x_node_k, x_customer_j)] > machine_eps)
// //                 {
// //                     lhs_value += x[var_x(x_node_k, x_customer_j)];
// //                     rmatind[index] = var_x(x_node_k, x_customer_j);
// //                     rmatval[index] = 0.5;
// //                     index++;
// //                 }
// //                 
// //                 if (x[var_x(x_node_k, x_customer_j)] > machine_eps)
// //                 {
// //                     lhs_value += x[var_x(x_customer_j, x_node_k)];
// //                     rmatind[index] = var_x(x_customer_j, x_node_k);
// //                     rmatval[index] = 0.5;
// //                     index++;
// //                 }
// //                 
// //             }
// //             
// //             if (x[var_x(index_x.D0, x_customer_j)] > machine_eps)
// //             {
// //                 lhs_value += x[var_x(index_x.D0, x_customer_j)];
// //                 rmatind[index] = var_x(index_x.D0, x_customer_j);
// //                 rmatval[index] = 0.5;
// //                 index++;
// //             }
// //             
// //             if (x[var_x(x_customer_j, index_x.D1)] > machine_eps)
// //             {
// //                 lhs_value += x[var_x(x_customer_j, index_x.D1)];
// //                 rmatind[index] = var_x(x_customer_j, index_x.D1);
// //                 rmatval[index] = 0.5;
// //                 index++;
// //             }
// //             
// //             cout << "entrada + salida " << customers_labels[j] << " = " << lhs_value << endl;
// //             cout << "y " << customers_labels[j] << "_* = " << x[var_y(y_customer_j, y_node_j)] << endl;
// //             lhs_value = lhs_value / 2.0;
// //             lhs_value += x[var_y(y_customer_j, y_node_j)];
// //             rmatind[index] = var_y(y_customer_j, y_node_j);
// //             rmatval[index] = 1.0;
// //             index++;
// //             
// //             cout << "lhs_value = " << lhs_value << endl;
// //             if (lhs_value > 1.0) // corta la solución
// //             {
// //                 cout << "la restriccion corta la solucion" << endl;
// //                 cuts_added = true;
// //                 sense[0] = 'L';
// //                 rhs[0] = 1.0;
// //                 
// //                 status = CPXaddrows(env, cmrsp_model, 0, 1 /*cindex*/, index, rhs, sense, rmatbeg,
// //                                     rmatind, rmatval, NULL, NULL /*rowname*/);
// //                 
// //                 if (status)
// //                 {
// //                     free(x);
// //                     char errmsg[__MSG_ERR_LEN];
// //                     CPXgeterrorstring (env, status, errmsg);
// //                     throw CMRSPException("Error agregando restricciones de 3er familia: "+string(errmsg));
// //                 }                
// //             }
// //             else
// //             {
// //                 cout << "la restriccion no corta la solucion" << endl;
// //             }
// //             
// //         }
// //         
// //     }
// //     
// // }
// 
// 
// // bool component_cut(CPXCENVptr env, void *cbdata, int wherefrom, void *cbhandle, int *useraction_p)
// // {
// //     bool add_cuts = false;
// //     int numcols = CPXgetnumcols(env, cmrsp_model);
// //     double*  x = new double[numcols];
// //     
// //     bool cuts_generated = false;
// //     vector<vector<int> > components;
// //     vector<int> n_component_customers;
// //     int index_x_customer = index_x.CUSTOMER;
// //     int index_y_customer = index_y.CUSTOMER;
// //     set<int> not_visited_customers;
// //     vector<bool> visited = vector<bool>(n_customers + n_steiners);
// //     int cust, n_cust;
// //     int x_source, x_target, y_source, y_target;
// //     
// //     int status = CPXgetcallbacknodex (env, cbdata, wherefrom, x, 0, numcols-1);
// //     if ( status ) 
// //     {
// //         cout << "Error obteniendo la solución de la relajación actual" << endl;
// //         goto TERMINATE;
// //     }
// //     
// //     
// //     /**************************************************************************
// //      * Busqueda de las componentes conexas que contienen clientes
// //      * aisladas del deposito
// //      **************************************************************************/
// //     for (int i = 0; i < n_customers; i++)
// //         not_visited_customers.insert(i);
// //     
// //     for (int i = 0; i < n_customers+n_steiners; i++)
// //         visited[i] = false;
// //     
// //     while (!not_visited_customers.empty())
// //     {
// //         vector<int> component = vector<int>();
// //         vector<int> visit_stack = vector<int>();
// //         cust = *not_visited_customers.begin();
// //         
// //         component.push_back(cust); n_cust = 1;
// //         not_visited_customers.erase(cust);
// //         visited[cust] = true;
// //         visit_stack.push_back(cust);
// //         while (visit_stack.size() > 0)
// //         {
// //             int source = visit_stack.back();
// //             visit_stack.pop_back();
// //             
// //             x_source = index_x_customer + source;
// //             y_source = index_y_customer + source;
// //             for (int target = 0; target < _V1; target++)
// //             {
// //                 x_target = index_x_customer + target;
// //                 y_target = index_y_customer + target;
// //                 
// //                 if (!visited[target])
// //                 {
// //                     if ( (x[var_x(x_source, index_x.D1)] > machine_eps) || (x[var_x(index_x.D0, x_source)] > machine_eps) )
// //                     {
// //                         cuts_generated = true;
// //                     }
// //                     
// //                     /* verifica si esta conectado por medio de una conexion anillo o estrella */
// //                     if ( (x[var_x(x_source, x_target)] > machine_eps) || (x[var_x(x_target, x_source)] > machine_eps) ||
// //                          ( (source < n_customers) && (x[var_y(y_source, y_target)] > machine_eps) ) ||
// //                          ( (target < n_customers) && (x[var_y(y_target, y_source)] > machine_eps) )
// //                        )
// //                     {
// //                         component.push_back(target);
// //                         visit_stack.push_back(target);
// //                         if (target < n_customers)
// //                         {
// //                             not_visited_customers.erase(target);
// //                             n_cust++;
// //                         }
// //                         visited[target] = true;
// //                     }
// //                     
// //                 }
// //             }
// //         }
// //         
// //         if (component.size() > 1)
// //         {
// //             components.push_back(component);
// //             n_component_customers.push_back(n_cust);
// //         }
// //     }
// //     
// //     /**************************************************************************
// //      * Agregando restricciones a la formulación
// //      **************************************************************************/
// //     if (cuts_generated)
// //     {
// //         vector<int> in_component(n_customers + n_steiners);
// //         for (int i = 0; i < n_customers + n_steiners; i++)
// //             in_component[i] = -1;
// //         
// //         for (int k = 0; k < components.size(); k++)
// //         {
// //             for (int i = 0; i < components[k].size(); i++)
// //             {
// //                 in_component[components[k][i]] = k;
// //             }
// //         }
// //         
// //         int numnz = 2*_V*_V;
// //         int rmatind[numnz];
// //         double rmatval[numnz];
// //         double rhs;
// //         char sense;
// //         int index = 0;
// //         double lhs_value = 0.0;
// //         int nz;
// // 
// //         for (int k = 0; k < components.size(); k++)
// //         {
// //             index = 0;
// //             for (int i = 0; i < components[k].size(); i++)
// //             {
// //                 for (int j = 0; j < _V1; j++)
// //                 {
// //                     if (in_component[j] != k)
// //                     {
// //                         lhs_value += x[var_x(index_x_customer + components[k][i], index_x_customer + j)];
// //                         rmatind[index] = var_x(index_x_customer + components[k][i], index_x_customer + j);
// //                         rmatval[index] = 1.0;
// //                         index++;
// //                         
// //                         if (components[k][i] < n_customers) // i es cliente
// //                         {
// //                             lhs_value += x[var_y(index_y_customer + components[k][i], index_y_customer + j)];
// //                             rmatind[index] = var_y(index_y_customer + components[k][i], index_y_customer + j);
// //                             rmatval[index] = 1.0;
// //                             index++;
// //                         }
// //                     }
// //                 }
// //                 lhs_value += x[var_x(index_x_customer + components[k][i], index_x.D1)];
// //                 rmatind[index] = var_x(index_x_customer + components[k][i], index_x.D1);
// //                 rmatval[index] = 1.0;
// //                 index++;
// //                 
// //             }
// //             
// //             if (lhs_value > ceil( ((double)n_component_customers[k])/ring_capacity )) // corta la solución actual
// //             {
// //                 add_cuts = true;
// //                 sense = 'G';
// //                 rhs = ceil( ((double)n_component_customers[k])/ring_capacity );
// //                 status = CPXcutcallbackadd(env, cbdata, wherefrom, index, rhs, sense, rmatind, rmatval);
// //                 if ( status ) {
// //                     cout << "Error al intentar agregar cortes" << endl;
// //                     goto TERMINATE;
// //                 } 
// // 
// //             }
// //             
// //         }
// //         
// // /*        cout << "COMPONENTES: " << endl;
// //         for (int k = 0; k < components.size(); k++)
// //         {
// //             vector<int> component = components[k];
// //             cout << "COMPONENTE: " << endl;
// //             for (int i = 0; i < component.size(); i++)
// //             {
// //                 if (component[i] < n_customers)
// //                     cout << customers_labels[component[i]] << " ";
// //                 else
// //                     cout << steiners_labels[component[i]-n_customers] << " ";
// //             }
// //             cout << endl << "----------" << endl;
// //         }*/
// //         
// //     }        
// //     
// //     if (add_cuts)
// //     {
// //         cout << "se agregaron cortes familia 1" << endl;
// //     }
// // 
// // TERMINATE:
// //     delete[] x;
// //     return add_cuts;
// // }
// // 
// // 
// // 
// // 
// // 
// // 
