#include "SeparacionSumaUno.h"
#include "CMRSRep.h"
#include "CMRSPException.h"
#include "util.h"

#include <vector>
#include <set>
#include <cmath>
#include <iostream>
using namespace std;


SeparacionSumaUno::SeparacionSumaUno(CMRSRep* cmrs_rep)
{
    this->cmrs_rep = cmrs_rep;
    _cortes_aplicados = 0;
}

string SeparacionSumaUno::nombre_familia()
{
    return "suma_uno";
}

bool SeparacionSumaUno::separar(CPXCENVptr env, double* x, void *cbdata, int wherefrom) throw(CMRSPException)
{    
    /**************************************************************************
     * Busqueda de las componentes conexas que contienen clientes
     * aislados del deposito
     **************************************************************************/
    bool cuts_added = false;
    _cortes_aplicados = 0;
    
    int n_rings = cmrs_rep->n_rings();
    int ring_capacity = cmrs_rep->ring_capacity();
    int n_customers = cmrs_rep->n_customers();
    int n_steiners = cmrs_rep->n_steiners();
    v_index index_connection_cost = cmrs_rep->index_connection_cost();
    v_index index_routing_cost = cmrs_rep->index_routing_cost();
    v_index index_x = cmrs_rep->index_x();
    v_index index_y = cmrs_rep->index_y();
    v_index index_f = cmrs_rep->index_f();
    v_index index_z = cmrs_rep->index_z();
    vector<vector<double> > connection_cost = cmrs_rep->connection_cost();
    vector<vector<double> > routing_cost = cmrs_rep->routing_cost();
    int _V = cmrs_rep->V();
    int _V1 = cmrs_rep->V1();
    

    /**************************************************************************
     * ACTUALIZAR !!!!!!!!!
     * 
     * Si para algun par u \in U, v \in V', se cumple:
     * \sum_{v' in V', v' \neq v} x_{u v'} + y_{u v'} = 0
     * 
     * Entonces debe ser: 
     * x_{u v} + y_{u v} = 1 porque \sum_{v' in V'} x_{u v'} + y_{u v'} = 1
     * 
     * Como además: x_{u v} + x_{v u} \leq 1, entonces si y_{u v} = 0 debe ser:
     * x_{u v} = 1, x_{v u} = 0
     * 
     * Desigualdad valida:
     * x_{u' u} + y_{u' u} \leq \sum_{v' in V', v' \neq u'} x_{u v'} + y_{u v'}
     * 
     * Algoritmo de separacion:
     * Busca u v tales que x_{u' u} + y_{u' u} > 1 - x_{u u'} - y_{u u'}
     * (Ejemplo: x_{v u} = 0.7, x_{u v} = 0.4)
     * 
     * Demostracion: 
     * Como \sum_{v' in V'} x_{u v'} + y_{u v'} = 1
     * Entonces;
     * x_{u u'} + y_{u u'} + \sum_{v' in V' v' \neq v} x_{u v'} + y_{u v'} = 1
     * x_{u u'} + y_{u u'} = 1 - \sum_{v' in V' v' \neq v} x_{u v'} - y_{u v'}
     * 
     * Por lo tanto: 
     * x_{u' u} + y_{u' u} > 1 - (1 - \sum_{v' in V' v' \neq u'} x_{u v'} - y_{u v'}) = 
     *            = \sum_{v' in V' v' \neq u'} x_{u v'} + y_{u v'}
     * 
     * viola la desigualdad valida (corta la solucion)
     * ------------------------------------------------------------------------
     * 
     * Desigualdad valida:
     * ¿¿ x_{w u} \leq \sum_{v' in V', v' \neq w} x_{u v'} + y_{u v'} ??
     * 
     * Algoritmo de separacion:
     * 
     * Demostracion: 
     * 
     **************************************************************************/
    
    for (int u = 0; u < n_customers; u++)
    {
        for (int v = 0; v < n_customers; v++)
        {
            if (u == v) continue;
            
            if (x[cmrs_rep->var_x(index_x.CUSTOMER + v, index_x.CUSTOMER + u)] + x[cmrs_rep->var_y(index_y.CUSTOMER + v, index_y.CUSTOMER + u)] 
                   > 1.0 - x[cmrs_rep->var_x(index_x.CUSTOMER + u, index_x.CUSTOMER + v)]
                         - x[cmrs_rep->var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v)] + m_eps )
            {
                int numnz = 3 + 2*(_V1 - 1);
                int rmatind[numnz];
                double rmatval[numnz];
                double rhs_value = 0.0;
                
                rmatind[0] = cmrs_rep->var_x(index_x.CUSTOMER + v, index_x.CUSTOMER + u);
                rmatval[0] = 1.0;
                
                rmatind[1] = cmrs_rep->var_y(index_y.CUSTOMER + v, index_y.CUSTOMER + u);
                rmatval[1] = 1.0;
                
                rmatind[2] = cmrs_rep->var_x(index_x.CUSTOMER + u, index_x.D1);
                rmatval[2] = -1.0;
                
                int index = 3;
                for (int v1 = 0; v1 < _V1; v1++)
                {
                    if (v1 == v) continue;

                    rmatind[index] = cmrs_rep->var_x(index_x.CUSTOMER + u, index_x.CUSTOMER + v1);
                    rmatval[index] = -1.0;
                    index++;

                    rmatind[index] = cmrs_rep->var_y(index_y.CUSTOMER + u, index_y.CUSTOMER + v1);
                    rmatval[index] = -1.0;
                    index++;
                }

                cuts_added = true;
                
                int status = CPXcutcallbackadd (env, cbdata, wherefrom, numnz, rhs_value, 'L', rmatind, rmatval);
                if (status)
                {
                    char errmsg[__MSG_ERR_LEN];
                    CPXgeterrorstring (env, status, errmsg);
                    throw CMRSPException("Error agregando restricciones de familia suma_uno: "+string(errmsg));
                }
                
                _cortes_aplicados++;
            }
            
        }
    }
    
    return cuts_added;
}

int SeparacionSumaUno::cortes_aplicados()
{
    return _cortes_aplicados;
}
