#include <iostream>
#include <string>
#include <vector>
#include <cstdlib>
#include "ExecParam.h"
#include "CMRSPbranching.h"
#include "CMRS.h"
#include "CMRSRep.h"
#include "HeuristicaInicial.h"
#include "HeuristicaInicialClusterRoute1.h"
#include "Separacion.h"
#include "SeparacionComponentes.h"
//#include "SeparacionComponentes1.h"
//#include "SeparacionAnilloOEstrella.h"
//#include "SeparacionAnilloOEstrella1.h"
#include "SeparacionAnilloOEstrella2.h"
//#include "SeparacionBusquedaLocal.h"
//#include "SeparacionDistancia.h"
#include "SeparacionSumaUno.h"
using namespace std;

int string_to_int(const string&);

int main(int argc, char** argv)
{
    cout << "Capacitated m-Ring-Star Problem Solver" << endl;

    if (argc < 23)
    {
        cout << "Uso: cmrsopt instancia.dat node_sel var_sel var_priority mip_emphasis time_limit logging_interval initial_heuristic primal_heuristic clique_cuts cover_cuts disjuntive_cuts flowcover_cuts flowpath_cuts fractional_cuts gubcover_cuts mir_cuts user_branching user_cuts continuous_var sos romp_simetria" << endl;
        return -1;
    }
    
    try
    {
        ExecParam param;
        param.filename = string(argv[1]);
        param.node_sel = string_to_int(argv[2]);
        param.var_sel = string_to_int(argv[3]);
        param.var_priority = string_to_int(argv[4]);
        param.mip_emphasis = string_to_int(argv[5]);
        param.time_limit = string_to_int(argv[6]);
        param.logging_interval = string_to_int(argv[7]);
        param.initial_heuristic = string_to_int(argv[8]);
        param.primal_heuristic = string_to_int(argv[9]);
        param.clique_cuts = string_to_int(argv[10]);
        param.cover_cuts = string_to_int(argv[11]);
        param.disjuntive_cuts = string_to_int(argv[12]);
        param.flowcover_cuts = string_to_int(argv[13]);
        param.flowpath_cuts = string_to_int(argv[14]);
        param.fractional_cuts = string_to_int(argv[15]);
        param.gubcover_cuts = string_to_int(argv[16]);
        param.mir_cuts = string_to_int(argv[17]);
        param.branching_1 = string_to_int(argv[18]);
        param.cutting_plane_1 = string_to_int(argv[19]);
        param.continuous_vars = string_to_int(argv[20]);
        param.sos = string_to_int(argv[21]);
        param.romp_simetria = string_to_int(argv[22]);
        param.ipc = string_to_int(argv[23]);
        param.skip_factor = string_to_int(argv[24]);
        
//         param.node_sel = -1;
//         param.var_sel = -1;
//         param.var_priority = 0;
//         param.mip_emphasis = -1;
//         param.time_limit = string_to_int(argv[6]);
//         param.logging_interval = string_to_int(argv[7]);
//         param.initial_heuristic = 0;
//         param.primal_heuristic = string_to_int(argv[9]);
//         param.clique_cuts = 1;
//         param.cover_cuts = 1;
//         param.disjuntive_cuts = 1;
//         param.flowcover_cuts = 1;
//         param.flowpath_cuts = 1;
//         param.fractional_cuts = 1;
//         param.gubcover_cuts = 1;
//         param.mir_cuts = 1;
//         param.branching_1 = 0;
//         param.cutting_plane_1 = 0;
//         param.continuous_vars = 0;
//         param.sos = 0;
//         param.romp_simetria = 0;

        CMRSRep cmrs_rep = CMRSRep(param.filename);
        CMRSP cmrsp(&cmrs_rep, param);
        
        HeuristicaInicial* heuristica_inicial = new HeuristicaInicialClusterRoute1(&cmrs_rep);
        Separacion* metodo_separacion_componentes = new SeparacionComponentes(&cmrs_rep);
        //Separacion* metodo_separacion_anillo_o_estrella = new SeparacionAnilloOEstrella(&cmrs_rep);
        //Separacion* metodo_separacion_anillo_o_estrella1 = new SeparacionAnilloOEstrella1(&cmrs_rep);
        Separacion* metodo_separacion_anillo_o_estrella2 = new SeparacionAnilloOEstrella2(&cmrs_rep);
        //Separacion* metodo_separacion_distancia = new SeparacionDistancia(&cmrs_rep);
        Separacion* metodo_separacion_suma_uno = new SeparacionSumaUno(&cmrs_rep);
        
        if (param.initial_heuristic)
        {
            cmrsp.set_heuristica_inicial(heuristica_inicial);
        }
        
        cout << "param.cutting_plane_1 = " << param.cutting_plane_1 << endl;
        cout << "param.node_sel = " << param.node_sel << endl;
        cout << "param.var_sel = " << param.var_sel << endl;
        cout << "param.ipc = " << param.ipc << endl;
        cout << "param.skip_factor = " << param.skip_factor << endl;
        cout << "param.initial_heuristic = " << param.initial_heuristic << endl;
        switch (param.cutting_plane_1)
        {
            case 1:
                cmrsp.add_metodo_separacion(metodo_separacion_componentes);
                break;
            case 2:
                cmrsp.add_metodo_separacion(metodo_separacion_anillo_o_estrella2);
                break;
            case 3:
                cmrsp.add_metodo_separacion(metodo_separacion_suma_uno);
                break;
            case 4:
                cmrsp.add_metodo_separacion(metodo_separacion_componentes);
                cmrsp.add_metodo_separacion(metodo_separacion_anillo_o_estrella2);
                break;
            case 5:
                cmrsp.add_metodo_separacion(metodo_separacion_componentes);
                cmrsp.add_metodo_separacion(metodo_separacion_suma_uno);
                break;
            case 6:
                cmrsp.add_metodo_separacion(metodo_separacion_anillo_o_estrella2);
                cmrsp.add_metodo_separacion(metodo_separacion_suma_uno);
                break;
            case 7:
                cmrsp.add_metodo_separacion(metodo_separacion_componentes);
                cmrsp.add_metodo_separacion(metodo_separacion_anillo_o_estrella2);
                cmrsp.add_metodo_separacion(metodo_separacion_suma_uno);
                break;
        }
        
        cmrsp.optimize();
        cmrsp.save_log();
        CMRS cmrs = cmrsp.solution();
        cmrs.save(param.filename+".sol");
        
        delete heuristica_inicial;
        delete metodo_separacion_componentes;
        //delete metodo_separacion_anillo_o_estrella;
        //delete metodo_separacion_anillo_o_estrella1;
        delete metodo_separacion_anillo_o_estrella2;
        //delete metodo_separacion_distancia;
        delete metodo_separacion_suma_uno;
    }
    catch(CMRSPException& e)
    {        
        cout << "**** ERROR ****" << endl << e.what() << endl;
    }

    return 0;
}

int string_to_int(const string& s)
{
    return atoi(s.c_str());
}

