#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################
##
## Convierte las instancias de CmRSP (Capacitated m-Ring Star Problem) de Edna a formato de entrada para el programa de optimización
## 
## Parametros del programa:
## archivo = archivo de entrada con el formato de Edna
##
## Uso: convert.py archivo
## Ejemplo de uso: convert.py edna-instances-cmrsp/BDS/eil26.tsp.3.6.3.A.BDS.cmrsp
## convert.py edna-instances-cmrsp/BDS/eil26.tsp.3.12.5.A.BDS.cmrsp
## convert.py edna-instances-cmrsp/BDS/eil26.tsp.3.12.5.B.BDS.cmrsp
## convert.py eil26.tsp.3.12.5.A.BDS.cmrsp
## convert.py eil26.tsp.3.12.5.B.BDS.cmrsp
##
################################################################################

import sys
import random
import math
import parametros as param


if len(sys.argv) != 2:
    print 'Uso: convert.py archivo'
    sys.exit(1)

archivo = sys.argv[1]

f = open(archivo, 'r') # f = open('edna-instances-cmrsp/BDS/eil26.tsp.3.6.3.A.BDS.cmrsp', 'r')

line = f.readline()
n, n_clientes, n_steiners, q, m = [int(k) for k in line[:-1].split(' ')]
lines = [line for line in f.readlines() if line[0] == 'a' or line[0] == 'e']

f.close()

c = {}
d = {}

for line in lines:
    if line[-1] == '\n':
        line = line[:-1]
    tipo = line[0]
    if tipo == 'a' or tipo == 'e':
        line = line.split(' ')
        origen, destino, costo = int(line[1]), int(line[2]), float(line[3])
        if tipo == 'a':
            d[(origen, destino)] = costo
            d[(destino, origen)] = costo
        elif tipo == 'e':
            c[(origen, destino)] = costo
            c[(destino, origen)] = costo


M = sum(d.values()) + sum(c.values()) + 1

f = open(archivo + '.dat', 'w')
f.write(str(m)+'\n')
f.write(str(q)+'\n')
f.write(str(n_clientes)+'\n')
f.write(str(n_steiners)+'\n')
f.write('d0\n')
f.write('d1\n')

for i in range(n_clientes):
    f.write('u'+str(i)+'\n')

for i in range(n_steiners):
    f.write('w'+str(i)+'\n')

for i in range(n_clientes+n_steiners+1):
    line = ''
    for j in range(n_clientes+n_steiners+1):
        if (i, j) in c:
            line = line + str(c[(i, j)]) + ' '
        elif i == j:
            line = line + '0.0 '
        else:
            line = line + str(M) + ' '
        
        if j == 0:
            line = line + line
        
    line = line[:-1] + '\n'
    if i == 0:
        line = line + line
    f.write(line)

for i in range(1, n_clientes+1):
    line = ''
    for j in range(1, n_clientes+n_steiners+1):
        if (i, j) in d:
            line = line + str(d[(i, j)]) + ' '
        elif i == j:
            line = line + '0.0 '
        else:
            line = line + str(M) + ' '
    
    line = line[:-1] + '\n'
    f.write(line)    

f.close()



