#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################
## Generador de instancias de CmRSP (Capacited m-Ring Star Problem)
## 
## Descripción: 
## Crea un archivo que contiene información de parámetros del problema y 
## matrices de costo de conexión y costo de ruteo.
##
## Parametros del programa:
## m = cantidad de anillos
## Q = cantidad máxima de clientes por anillo
## U = cantidad de nodos cliente
## W = cantidad de nodos de transicion
##
## Uso: gen.py m Q U W g
## Donde: g es la cantidad de instancias a generar
## Ejemplo de uso: gen.py 3 5 12 3 10
################################################################################


################################################################################
## Formato del archivo de salida:
## M
## Q
## U
## W
## d0 (nombre d0)
## d1 (nombre d1)
## u1 (nombre cliente 1)
## .. 
## uU (nombre cliente U)
## w1 (nombre del nodo de tránsito 1)
## ..
## wW (nombre del nodo de tránsito W)
## C (matriz de costos de conexion, (U+W)x(U+W))
## D (matriz de costos de ruteo, Ux(U+W))
################################################################################

import sys
import random

if len(sys.argv) != 6:
    print 'Uso : gen.py m q u w n'
    print 'm cantidad de anillos'
    print 'q capacidad del anillo (cantidad maxima de clientes conectados al anillo)'
    print 'u cantidad de clientes'
    print 'w cantidad de nodos de transicion'
    print 'n cantidad de instancias a generar'
    sys.exit(1)

M = int(sys.argv[1])
Q = int(sys.argv[2])
U = int(sys.argv[3])
W = int(sys.argv[4])
N = int(sys.argv[5])

if M*Q < U:
    print 'Error: No puede ser m*q < u'
    sys.exit(1)

min_cost = 100
max_cost = 200

for i in range(N):
    archivo = 'cmrsp-m'+str(M)+'q'+str(Q)+'u'+str(U)+'w'+str(W)+'_'+str(i)+'.dat'
    f = open(archivo, 'w')
    f.write(str(M)+'\n')
    f.write(str(Q)+'\n')
    f.write(str(U)+'\n')
    f.write(str(W)+'\n')
    
    f.write('d0\n')
    f.write('d1\n')
    
    for u in range(U):
        f.write('u'+str(u)+'\n')
    
    for w in range(W):
        f.write('w'+str(w)+'\n')    
    
    for j in range(U+W+2):
        for k in range(U+W+2):
            f.write(str(int(random.uniform(min_cost, max_cost)))+'.0 ')
        
        f.write('\n')
    
    for j in range(U):
        for k in range(U+W):
            f.write(str(int(random.uniform(min_cost, max_cost)))+'.0 ')
        
        f.write('\n')
    
    f.close()

