#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################
##
## Generador de instancias de CmRSP (Capacitated m-Ring Star Problem)
## 
## Descripción: 
## Crea un archivo que contiene información de parámetros del problema y 
## matrices de costo de conexión y costo de ruteo.
##
## Parametros del programa:
## m = cantidad de anillos
## Q = cantidad máxima de clientes por anillo
## U = cantidad de nodos cliente
## W = cantidad de nodos de transicion
##
## Uso: gen_inst.py num_instancia m Q U W
## Ejemplo de uso: gen_inst.py 1 3 5 12 3
##
################################################################################

import sys
import random
import math
import parametros as param


def guardar_mapa(deposito,  clientes,  steiners, num_instancia, m, q):
    #f = open(param.archivo_mapa % (num_instancia, n, m, len(clientes), len(steiners), q),  'w')
    f = open(param.nombre_archivo(param.archivo_mapa, num_instancia, m, len(clientes), len(steiners), q),  'w')
    
    f.write(str(len(clientes))+'\n')
    f.write(str(len(steiners))+'\n')
    
    deposito_label,  deposito_x,  deposito_y = deposito
    f.write(deposito_label+'0_'+deposito_label+'1'+'\t'+str(deposito_x)+'\t'+str(deposito_y)+'\n')
    
    for cliente_label,  cliente_x,  cliente_y in clientes:
        f.write(cliente_label+'\t'+str(cliente_x)+'\t'+str(cliente_y)+'\n')
    
    for steiner_label,  steiner_x, steiner_y in steiners:
        f.write(steiner_label+'\t'+str(steiner_x)+'\t'+str(steiner_y)+'\n')
    
    f.close()


def guardar_instancia(m,  q, deposito_label, clientes_labels, steiners_labels, costos_conexion, costos_ruteo,  num_instancia):
    n = len(clientes_labels)+len(steiners_labels)
    #f = open(param.archivo_instancia % (num_instancia, n, m, len(clientes_labels), len(steiners_labels), q), 'w')
    f = open(param.nombre_archivo(param.archivo_instancia, num_instancia, m, len(clientes_labels), len(steiners_labels), q), 'w')
    
    f.write(str(m)+'\n')
    f.write(str(q)+'\n')
    f.write(str(len(clientes_labels))+'\n')
    f.write(str(len(steiners_labels))+'\n')
    f.write(deposito_label+'0\n')
    f.write(deposito_label+'1\n')
    
    for cliente_label in clientes_labels:
        f.write(cliente_label+'\n')
    
    for steiner_label in steiners_labels:
        f.write(steiner_label+'\n')
    
    for costo_conexion_fila in costos_conexion:
        f.write(reduce(lambda x, y: str(x)+' '+str(y),  costo_conexion_fila)+'\n')
    
    for costo_ruteo_fila in costos_ruteo:
        f.write(reduce(lambda x, y: str(x)+' '+str(y),  costo_ruteo_fila)+'\n')
    
    f.close()


def distancia_euclidea(vector1,  vector2):
    return round( math.sqrt( (vector1[1] - vector2[1])**2 + (vector1[2] - vector2[2])**2 ) )

def calcular_costos(deposito,  clientes,  steiners):
    costos_conexion = []
    costos_ruteo = []
    nodos = [deposito, deposito] + clientes + steiners
    for n in nodos:
        costos_conexion_fila = []
        for m in nodos:
            d = distancia_euclidea(n, m)
            costos_conexion_fila.append(d)
        costos_conexion.append(costos_conexion_fila)
    
    for n in clientes:
        costos_ruteo_fila = []
        for m in clientes+steiners:
            d = distancia_euclidea(n, m)
            costos_ruteo_fila.append(d)
        costos_ruteo.append(costos_ruteo_fila)
    
    return costos_conexion, costos_ruteo


def generar_instancia(num_instancia, num_clientes, num_steiners, param_anillos):
    # Generación de posición de nodo de depósito
    deposito_label = 'D'
    deposito_x = random.sample(xrange(param.RANDOM_RANGE), 1)
    deposito_y = random.sample(xrange(param.RANDOM_RANGE), 1)
    deposito = (deposito_label,  deposito_x[0],  deposito_y[0])
    
    # Generación de posiciones de nodos de Clientes
    clientes_labels = ['u'+str(i) for i in range(num_clientes)]
    clientes_x = random.sample(xrange(param.RANDOM_RANGE),  num_clientes)
    clientes_y = random.sample(xrange(param.RANDOM_RANGE),  num_clientes)
    clientes = [(clientes_labels[i],  clientes_x[i],  clientes_y[i]) for i in range(num_clientes)]
    
    # Generación de posiciones de nodos de Steiner
    steiners_labels = ['w'+str(i) for i in range(num_steiners)]
    steiners_x = random.sample(xrange(param.RANDOM_RANGE),  num_steiners)
    steiners_y = random.sample(xrange(param.RANDOM_RANGE),  num_steiners)
    steiners = [(steiners_labels[i],  steiners_x[i],  steiners_y[i]) for i in range(num_steiners)]
    
    # Calcula los costos de conexión y ruteo (usa la distancia euclidea como función de costo)
    costos_conexion, costos_ruteo = calcular_costos(deposito, clientes, steiners)
    
    for param_anillo in param_anillos:
        # Guarda el mapa con las posiciones del deposito, clientes y nodos de steiner
        guardar_mapa(deposito,  clientes,  steiners,  num_instancia, param_anillo[0], param_anillo[1])
        
        # Guarda la instancia del problema que se utiliza como entrada para el modelo de optimización
        guardar_instancia(param_anillo[0], param_anillo[1], deposito_label, clientes_labels, steiners_labels, costos_conexion, costos_ruteo, num_instancia=num_instancia)


num_instancia = 1
num_anillos = [2,3,4,5]

for num_clientes in [8,9,10,11,12,13,14]:
    for num_steiners in [0,2,4]:
        param_anillos = [(m,q) for m in num_anillos for q in range( int(math.ceil(1.0*num_clientes/m)), int(math.ceil(1.0*num_clientes/(m-1)))+1 ) if q < num_clientes and (m-1)*q < num_clientes ]
        
        for instancias in range(20):
            generar_instancia(num_instancia, num_clientes, num_steiners, param_anillos)
            num_instancia = num_instancia + 1

