#!/usr/bin/python
# -*- coding: utf-8 -*-

import time
import os
#import subprocess

################################################################################
# Configuracion
################################################################################

# Archivos de instancias
DIR_INSTANCES = './edna-instances/BDS'

INSTANCES_26 = [ 'eil26.tsp.3.12.5.A.BDS.cmrsp.dat' , 
                    'eil26.tsp.4.12.4.A.BDS.cmrsp.dat', 
                    'eil26.tsp.5.12.3.A.BDS.cmrsp.dat', 
                    'eil26.tsp.3.18.7.A.BDS.cmrsp.dat', 
                    'eil26.tsp.4.18.5.A.BDS.cmrsp.dat', 
                    'eil26.tsp.5.18.4.A.BDS.cmrsp.dat', 
                    'eil26.tsp.3.25.10.A.BDS.cmrsp.dat', 
                    'eil26.tsp.4.25.7.A.BDS.cmrsp.dat', 
                    'eil26.tsp.5.25.6.A.BDS.cmrsp.dat', ]

INSTANCES_51 = [ 'eil51.tsp.3.12.5.A.BDS.cmrsp.dat' , 
                    'eil51.tsp.4.12.4.A.BDS.cmrsp.dat', 
                    'eil51.tsp.5.12.3.A.BDS.cmrsp.dat', 
                    'eil51.tsp.3.25.10.A.BDS.cmrsp.dat', 
                    'eil51.tsp.4.25.7.A.BDS.cmrsp.dat', 
                    'eil51.tsp.5.25.6.A.BDS.cmrsp.dat', ]

DIR_INSTANCES = './hernan-instances'
HERNAN_INSTANCES = os.listdir('./hernan-instances')

INSTANCES = HERNAN_INSTANCES

# Archivo de log
DEFAULT_LOG_FILE = 'tests.log'

# Formato de 
format_localtime = lambda dt: str(dt[0])+'-'+str(dt[1]).zfill(2)+'-'+str(dt[2]).zfill(2)+'_'+str(dt[3]).zfill(2)+':'+str(dt[4]).zfill(2)


################################################################################
# Parametros para resolver una instancia
################################################################################
class Param:
    def __init__(self):
        self.node_sel = None
        self.var_sel = None
        self.var_priority = None
        self.mip_emphasis = None
        self.time_limit = None
        self.logging_interval = None
        self.initial_heuristic = None
        self.primal_heuristic = None
        self.clique_cuts = None
        self.cover_cuts = None
        self.disjuntive_cuts = None
        self.flowcover_cuts = None
        self.flowpath_cuts = None
        self.fractional_cuts = None
        self.gubcover_cuts = None
        self.mir_cuts = None
        self.user_branching = None
        self.user_cuts = None
        self.continuous_var = None
        self.sos = None
        self.romp_simetria = None

################################################################################
# Valores de parametros por defecto
################################################################################
def get_default_param():
    default_param = Param()
    default_param.node_sel = 1
    default_param.var_sel = 4
    default_param.var_priority = 3
    default_param.mip_emphasis = 1
    default_param.time_limit = 1200
    default_param.logging_interval = 100
    default_param.initial_heuristic = 1
    default_param.primal_heuristic = 0
    default_param.clique_cuts = 1
    default_param.cover_cuts = 1
    default_param.disjuntive_cuts = 1
    default_param.flowcover_cuts = 1
    default_param.flowpath_cuts = 1
    default_param.fractional_cuts = 1
    default_param.gubcover_cuts = 1
    default_param.mir_cuts = 1
    default_param.user_branching = 1
    default_param.user_cuts = 1
    default_param.continuous_var = 0
    default_param.sos = 0
    default_param.romp_simetria = 0
    return default_param


################################################################################
# CPLEX default
################################################################################
def get_test_params_cplex_default(default_param):
    test_params = []
    param = Param()
    param.node_sel = -1
    param.var_sel = -1
    param.var_priority = 0
    param.mip_emphasis = -1
    param.time_limit = 1200
    param.logging_interval = 100
    param.initial_heuristic = 0
    param.primal_heuristic = 0
    param.clique_cuts = -1
    param.cover_cuts = -1
    param.disjuntive_cuts = -1
    param.flowcover_cuts = -1
    param.flowpath_cuts = -1
    param.fractional_cuts = -1
    param.gubcover_cuts = -1
    param.mir_cuts = -1
    param.user_branching = 0
    param.user_cuts = 0
    param.continuous_var = 0
    param.sos = 0
    param.romp_simetria = 0
    test_params.append(param)
    
    return test_params

################################################################################
# CPLEX default + Heuristica inicial
################################################################################
def get_test_params_cplex_default_heuristica_inicial(default_param):
    test_params = []
    for initial_heuristic in [0,1]:
        param = Param()
        param.node_sel = -1
        param.var_sel = -1
        param.var_priority = 0
        param.mip_emphasis = -1
        param.time_limit = 3600
        param.logging_interval = 100
        param.initial_heuristic = initial_heuristic
        param.primal_heuristic = 0
        param.clique_cuts = -1
        param.cover_cuts = -1
        param.disjuntive_cuts = -1
        param.flowcover_cuts = -1
        param.flowpath_cuts = -1
        param.fractional_cuts = -1
        param.gubcover_cuts = -1
        param.mir_cuts = -1
        param.user_branching = 0
        param.user_cuts = 0
        param.continuous_var = 0
        param.sos = 0
        param.romp_simetria = 0
        test_params.append(param)
    
    return test_params

################################################################################
# CPLEX default + Rompimiento de simetria Modo 1
################################################################################
def get_test_params_cplex_default_rompimiento_simetria_1(default_param):
    test_params = []
    param = Param()
    param.node_sel = -1
    param.var_sel = -1
    param.var_priority = 0
    param.mip_emphasis = -1
    param.time_limit = 3600
    param.logging_interval = 100
    param.initial_heuristic = 1
    param.primal_heuristic = 0
    param.clique_cuts = -1
    param.cover_cuts = -1
    param.disjuntive_cuts = -1
    param.flowcover_cuts = -1
    param.flowpath_cuts = -1
    param.fractional_cuts = -1
    param.gubcover_cuts = -1
    param.mir_cuts = -1
    param.user_branching = 0
    param.user_cuts = 0
    param.continuous_var = 0
    param.sos = 0
    param.romp_simetria = 1
    test_params.append(param)
    
    return test_params

################################################################################
# CPLEX default + Rompimiento de simetria Modo 2
################################################################################
def get_test_params_cplex_default_rompimiento_simetria_2(default_param):
    test_params = []
    param = Param()
    param.node_sel = -1
    param.var_sel = -1
    param.var_priority = 0
    param.mip_emphasis = -1
    param.time_limit = 3600
    param.logging_interval = 100
    param.initial_heuristic = 1
    param.primal_heuristic = 0
    param.clique_cuts = -1
    param.cover_cuts = -1
    param.disjuntive_cuts = -1
    param.flowcover_cuts = -1
    param.flowpath_cuts = -1
    param.fractional_cuts = -1
    param.gubcover_cuts = -1
    param.mir_cuts = -1
    param.user_branching = 0
    param.user_cuts = 0
    param.continuous_var = 0
    param.sos = 0
    param.romp_simetria = 2
    test_params.append(param)
    
    return test_params

################################################################################
# CPLEX default 
# 1. Sin rompimiento de simetria
# 2. Con rompimiento de simetria modo 1
# 3. Con rompimiento de simetria modo 2
################################################################################
def get_test_params_cplex_default_rompimiento_simetria(default_param):
    test_params = []
    for romp_simetria in [0, 1, 2]:
        param = Param()
        param.node_sel = -1
        param.var_sel = -1
        param.var_priority = 0
        param.mip_emphasis = -1
        param.time_limit = 3600
        param.logging_interval = 100
        param.initial_heuristic = 1
        param.primal_heuristic = 0
        param.clique_cuts = -1
        param.cover_cuts = -1
        param.disjuntive_cuts = -1
        param.flowcover_cuts = -1
        param.flowpath_cuts = -1
        param.fractional_cuts = -1
        param.gubcover_cuts = -1
        param.mir_cuts = -1
        param.user_branching = 0
        param.user_cuts = 0
        param.continuous_var = 0
        param.sos = 0
        param.romp_simetria = romp_simetria
        test_params.append(param)
    
    return test_params


################################################################################
# Un conjunto de valores de parametros para pruebas
################################################################################
def get_test_params(default_param):
    test_params = []
    for node_sel in [3]:
        for mip_emphasis in [1]:
            for var_sel in [4]:
                for var_priority in [1, 2, 3, 4]:
                    for user_cuts in [1, 2, 3]:
                        for user_branching in range(2):
                            param = Param()
                            param.node_sel = node_sel
                            param.var_sel = var_sel
                            param.var_priority = var_priority
                            param.mip_emphasis = mip_emphasis
                            param.time_limit = default_param.time_limit
                            param.logging_interval = default_param.logging_interval
                            param.initial_heuristic = default_param.initial_heuristic
                            param.primal_heuristic = default_param.primal_heuristic
                            param.clique_cuts = default_param.clique_cuts
                            param.cover_cuts = default_param.cover_cuts
                            param.disjuntive_cuts = default_param.disjuntive_cuts
                            param.flowcover_cuts = default_param.flowcover_cuts
                            param.flowpath_cuts = default_param.flowpath_cuts
                            param.fractional_cuts = default_param.fractional_cuts
                            param.gubcover_cuts = default_param.gubcover_cuts
                            param.mir_cuts = default_param.mir_cuts
                            param.user_branching = user_branching
                            param.user_cuts = user_cuts
                            param.continuous_var = default_param.continuous_var
                            param.sos = default_param.sos
                            param.romp_simetria = default_param.romp_simetria
                            test_params.append(param)

    return test_params


################################################################################
# Conjunto de valores de parametros para la prueba de Heuristica Inicial
################################################################################
def get_test_params_initial_heuristic(default_param):
    test_params = []
    for node_sel in [3]:
        for mip_emphasis in [1]:
            for var_sel in [4]:
                for var_priority in [0]:
                    for initial_heuristic in [0, 1]:
                        for user_cuts in [0]:
                            for user_branching in [0]:
                                param = Param()
                                param.node_sel = node_sel
                                param.var_sel = var_sel
                                param.var_priority = var_priority
                                param.mip_emphasis = mip_emphasis
                                param.time_limit = default_param.time_limit
                                param.logging_interval = default_param.logging_interval
                                param.initial_heuristic = initial_heuristic
                                param.primal_heuristic = default_param.primal_heuristic
                                param.clique_cuts = default_param.clique_cuts
                                param.cover_cuts = default_param.cover_cuts
                                param.disjuntive_cuts = default_param.disjuntive_cuts
                                param.flowcover_cuts = default_param.flowcover_cuts
                                param.flowpath_cuts = default_param.flowpath_cuts
                                param.fractional_cuts = default_param.fractional_cuts
                                param.gubcover_cuts = default_param.gubcover_cuts
                                param.mir_cuts = default_param.mir_cuts
                                param.user_branching = user_branching
                                param.user_cuts = user_cuts
                                param.continuous_var = default_param.continuous_var
                                param.sos = default_param.sos
                                param.romp_simetria = default_param.romp_simetria
                                test_params.append(param)
    
    return test_params

################################################################################
# Conjunto de valores de parametros para la prueba de planos de corte Separacion
################################################################################
def get_test_params_user_cuts_separacion(default_param):
    test_params = []
    for node_sel in [3]:
        for mip_emphasis in [1]:
            for var_sel in [4]:
                for var_priority in [0]:
                    for initial_heuristic in [0]:
                        for user_cuts in [4]:
                            for user_branching in [0]:
                                param = Param()
                                param.node_sel = node_sel
                                param.var_sel = var_sel
                                param.var_priority = var_priority
                                param.mip_emphasis = mip_emphasis
                                param.time_limit = default_param.time_limit
                                param.logging_interval = default_param.logging_interval
                                param.initial_heuristic = initial_heuristic
                                param.primal_heuristic = default_param.primal_heuristic
                                param.clique_cuts = default_param.clique_cuts
                                param.cover_cuts = default_param.cover_cuts
                                param.disjuntive_cuts = default_param.disjuntive_cuts
                                param.flowcover_cuts = default_param.flowcover_cuts
                                param.flowpath_cuts = default_param.flowpath_cuts
                                param.fractional_cuts = default_param.fractional_cuts
                                param.gubcover_cuts = default_param.gubcover_cuts
                                param.mir_cuts = default_param.mir_cuts
                                param.user_branching = user_branching
                                param.user_cuts = user_cuts
                                param.continuous_var = default_param.continuous_var
                                param.sos = default_param.sos
                                param.romp_simetria = default_param.romp_simetria
                                test_params.append(param)
    
    return test_params


################################################################################
# Logging con la informacion de la resolucion de una instancia:
# Archivo de la instancia, fecha de ejecucion, tiempo de ejecucion y
# parametros de entrada
################################################################################
def log_line(filename, param, exec_date, diff_time):
    header_params = ['filename', 'exec_date', 'time(seg)', 'time_limit', 'node_sel', 'var_sel', 'var_priority', 'mip_emphasis', 'initial_heuristic',
                                    'primal_heuristic',  'user_branching', 'user_cuts', 'continuous_var', 'clique_cuts', 'cover_cuts', 'disjuntive_cuts', 'flowcover_cuts', 
                                    'flowpath_cuts', 'fractional_cuts', 'gubcover_cuts', 'mir_cuts', 'sos', 'romp_simetria']
    header_line = reduce(lambda s, t: str(s)+' '+str(t), header_params)
    param_values = [filename, exec_date, diff_time, param.time_limit, param.node_sel, param.var_sel, param.var_priority, param.mip_emphasis, param.initial_heuristic, 
                                param.primal_heuristic, param.user_branching, param.user_cuts, param.continuous_var, param.clique_cuts, param.cover_cuts, param.disjuntive_cuts, 
                                param.flowcover_cuts, param.flowpath_cuts, param.fractional_cuts, param.gubcover_cuts, param.mir_cuts, param.sos, param.romp_simetria]
    line = reduce(lambda s, t: str(s)+' '+str(t), param_values)
    return line, header_line


################################################################################
# Escribe el logging en un archivo de salida
################################################################################
def write_to_log(line, header_line, log_file=None):
    if log_file == None:
        log_file = DEFAULT_LOG_FILE
    
    if not os.path.exists(log_file):
        f = open(log_file, 'w')
        f.write(header_line+'\n')
    else:
        f = open(log_file, 'a')
    
    f.write(line+'\n')
    f.close()


################################################################################
# Crea una lista de valores de parametros a string para pasarlos
# por linea de comando al optimizador
################################################################################
def param_to_cmdline_args(param):
    list_param = []
    list_param.append(param.node_sel)
    list_param.append(param.var_sel)
    list_param.append(param.var_priority)
    list_param.append(param.mip_emphasis)
    list_param.append(param.time_limit)
    list_param.append(param.logging_interval)
    list_param.append(param.initial_heuristic)
    list_param.append(param.primal_heuristic)
    list_param.append(param.clique_cuts)
    list_param.append(param.cover_cuts)
    list_param.append(param.disjuntive_cuts)
    list_param.append(param.flowcover_cuts)
    list_param.append(param.flowpath_cuts)
    list_param.append(param.fractional_cuts)
    list_param.append(param.gubcover_cuts)
    list_param.append(param.mir_cuts)
    list_param.append(param.user_branching)
    list_param.append(param.user_cuts)
    list_param.append(param.continuous_var)
    list_param.append(param.sos)
    list_param.append(param.romp_simetria)
    
    return map(str, list_param)


################################################################################
# Ejecuta el optimizador para la instancia y parametros especificados
# y escribe una linea de log (archivo de instancia, fecha de ejecucion,
# tiempo de ejecucion y parametros de entrada)
################################################################################
def execute_instance(filename, param, log_file):
    exec_date = format_localtime(time.localtime())
    start_time = time.time()
    cmdline_args = reduce(lambda s, t: s+' '+t, [filename]+param_to_cmdline_args(param))
    try:
        #subprocess.call(['cmrspopt', filename]+param_to_cmdline_args(param)) # En socrates hay una version vieja de python que no soporta la libreria subprocess
        print 'Ejecutando: cmrspopt  '+cmdline_args
        os.system('cmrspopt '+cmdline_args)
        #print 'cmrspopt '+cmdline_args
    except OSError:
        print 'Error ejecutando cmrspopt '+ cmdline_args
    
    end_time = time.time()
    diff_time = round(end_time - start_time, 0)
    line, header_line = log_line(filename, param, exec_date, diff_time)
    write_to_log(line, header_line, log_file)


################################################################################
# Ejecuta un conjunto de instancias de prueba con distintos parametros
# y escribe los resultados en un log
################################################################################
def cmp(x, y):
    int_x = int(x.split('.')[0].split('-')[2])
    int_y = int(y.split('.')[0].split('-')[2])
    if int_x < int_y:
        return -1
    elif int_x > int_y:
        return 1
    else:
        return 0


default_param = get_default_param()
#CPLEX_DEFAULT_TEST = (get_test_params_cplex_default(default_param), 'tests-01-hb-cplex_default.log')
#INITIAL_HEURISTIC_TEST = (get_test_params_initial_heuristic(default_param), 'tests-heuristica_inicial.log')
USER_CUTS_SEPARACION_TEST = (get_test_params_user_cuts_separacion(default_param), 'tests-01-hb-bc.log')
#CPLEX_DEFAULT_HEURISTICA_INICIAL_TEST = (get_test_params_cplex_default_heuristica_inicial(default_param), 'tests-01-hb-cplex_default_heuristica_inicial.log')
#CPLEX_DEFAULT_ROMPIMIENTO_SIMETRIA_TEST = (get_test_params_cplex_default_rompimiento_simetria(default_param), 'tests-01-hb-cplex_default_romp_sim.log')
#CPLEX_DEFAULT_ROMPIMIENTO_SIMETRIA_1_TEST = (get_test_params_cplex_default_rompimiento_simetria_1(default_param), 'tests-01-hb-cplex_default_romp_sim_1.log')
#CPLEX_DEFAULT_ROMPIMIENTO_SIMETRIA_2_TEST = (get_test_params_cplex_default_rompimiento_simetria_2(default_param), 'tests-01-hb-cplex_default_romp_sim_2.log')

test_params, log_file = USER_CUTS_SEPARACION_TEST

#DIR_INSTANCES = './hernan-instances'
#INSTANCES = os.listdir(DIR_INSTANCES)
#INSTANCES = filter(lambda x: x.find('cmrsp-inst-') > -1 and x.find('.dat') > -1, INSTANCES)
#INSTANCES.sort(cmp)
for file_instance in INSTANCES:
    for param in test_params:
        execute_instance(DIR_INSTANCES+'/'+file_instance, param, log_file)

