#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################################
##
## Visualizador de mapa de instancias y soluciones factibles de CmRSP (Capacitated m-Ring Star Problem)
## 
## Descripción: 
## xxxxxxxxxxxxxx
## 
## Uso: ver_mapa.py xxxxx
## Donde: xxxx
## Ejemplo de uso: ver_mapa.py xxxxx
##
## ver_mapa.py 1
## ver_mapa.py eil26.tsp.3.12.5.A.BDS.cmrsp.dat eil26.tsp.3.12.5.A.BDS.cmrsp.dat.sol
## ver_mapa.py eil26.tsp.3.12.5.B.BDS.cmrsp.dat eil26.tsp.3.12.5.B.BDS.cmrsp.dat.sol
################################################################################


import sys
import matplotlib
matplotlib.use('GTKAgg')
import pylab
from matplotlib import lines,  pyplot

import parametros as param


def abrir_mapa(archivo_mapa):
    f = open(archivo_mapa, 'r')
    num_clientes = int(f.readline())
    num_steiners = int(f.readline())
    deposito_label,  deposito_x,  deposito_y = f.readline()[:-1].split('\t')
    deposito_x = int(deposito_x)
    deposito_y = int(deposito_y)
    deposito = (deposito_label,  int(deposito_x),  int(deposito_y))
    clientes = []
    for i in range(num_clientes):
        cliente_label,  cliente_x,  cliente_y = f.readline()[:-1].split('\t')
        clientes.append((cliente_label,  int(cliente_x),  int(cliente_y)))
    
    steiners = []
    for i in range(num_steiners):
        steiner_label,  steiner_x,  steiner_y = f.readline()[:-1].split('\t')
        steiners.append((steiner_label,  int(steiner_x),  int(steiner_y)))
    
    f.close()
    return deposito,  clientes,  steiners

def abrir_solucion(archivo_solucion):
        f = open(archivo_solucion, 'r')
        num_conexiones_anillo = int(f.readline())
        num_conexiones_estrella = int(f.readline())
        conexiones_anillo = []
        for i in range(num_conexiones_anillo):
                source_label,  target_label = f.readline()[:-1].split('\t')
                conexiones_anillo.append((source_label,  target_label))
        
        conexiones_estrella = []
        for i in range(num_conexiones_estrella):
                source_label,  target_label = f.readline()[:-1].split('\t')
                conexiones_estrella.append((source_label,  target_label))
        
        f.close()
        return conexiones_anillo,  conexiones_estrella


def dibujar_mapa(plot, deposito,  clientes,  steiners):
    deposito_label,  deposito_x,  deposito_y = deposito
    pyplot.scatter([deposito_x],  [deposito_y],  marker='s')
    pyplot.text(deposito_x, deposito_y, deposito_label)
    
    clientes_labels = [cliente_label for cliente_label,  cliente_x,  cliente_y in clientes]
    clientes_x = [cliente_x for cliente_label,  cliente_x,  cliente_y in clientes]
    clientes_y = [cliente_y for cliente_label,  cliente_x,  cliente_y in clientes]
    pyplot.scatter(clientes_x,  clientes_y, marker='^')
    
    for cliente_label, cliente_x, cliente_y in clientes:
        pyplot.text(cliente_x, cliente_y, cliente_label)
    
    if steiners == []:
      return
    
    steiners_labels = [steiner_label for steiner_label,  steiner_x,  steiner_y in steiners]
    steiners_x = [steiner_x for steiner_label,  steiner_x,  steiner_y in steiners]
    steiners_y = [steiner_y for steiner_label,  steiner_x,  steiner_y in steiners]
    pyplot.scatter(steiners_x,  steiners_y, marker='o')
    for steiner_label, steiner_x, steiner_y in steiners:
        pyplot.text(steiner_x, steiner_y, steiner_label)


def dibujar_solucion(plot, deposito,  clientes,  steiners,  conexiones_anillo,  conexiones_estrella):
    coord_map = {}
    deposito_label,  deposito_x,  deposito_y = deposito
    coord_map[deposito_label] = (deposito_x, deposito_y)
    
    for cliente_label, cliente_x, cliente_y in clientes:
        coord_map[cliente_label] = (cliente_x, cliente_y)
    
    for steiner_label, steiner_x, steiner_y in steiners:
        coord_map[steiner_label] = (steiner_x, steiner_y)
    
    print coord_map
    print conexiones_anillo
    print conexiones_estrella
    
    conexiones_anillo_x = []
    conexiones_anillo_y = []
    
    for source_label, target_label in conexiones_anillo:
        conexion_anillo_x = [ coord_map[source_label][0],  coord_map[target_label][0] ]
        conexion_anillo_y = [ coord_map[source_label][1],  coord_map[target_label][1] ]
        plot.add_line(lines.Line2D(conexion_anillo_x,  conexion_anillo_y, linestyle='-'))
    
    for source_label, target_label in conexiones_estrella:
        conexion_estrella_x = [ coord_map[source_label][0],  coord_map[target_label][0] ]
        conexion_estrella_y = [ coord_map[source_label][1],  coord_map[target_label][1] ]
        plot.add_line(lines.Line2D(conexion_estrella_x,  conexion_estrella_y, linestyle='--'))



if len(sys.argv) <= 1 or len(sys.argv) >= 4:
    print 'Uso: ver_mapa.py [num_instancia | archivo_mapa archivo_solucion]'
    sys.exit(1)

if sys.argv[1].isdigit():
    num_instancia = int(sys.argv[1])
    archivo_mapa = None
    archivo_solucion = None
else:
    num_instancia = None
    archivo_mapa = sys.argv[1]
    archivo_solucion = sys.argv[2]

if archivo_mapa == None:
    archivo_mapa = param.archivo_mapa % num_instancia
    archivo_solucion = param.archivo_solucion % num_instancia

print 'archivo mapa: '+archivo_mapa
print 'archivo solución: '+archivo_solucion


deposito,  clientes,  steiners = abrir_mapa(archivo_mapa)
conexiones_anillo,  conexiones_estrella = abrir_solucion(archivo_solucion)

plot = pyplot.subplot(111)
dibujar_mapa(plot, deposito,  clientes,  steiners)
dibujar_solucion(plot, deposito,  clientes , steiners,  conexiones_anillo,  conexiones_estrella)
pyplot.show()

